# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .token import Token
from .token_canonical_sequences import (
    ALPHA_CHAIN_TOKEN_SEQ,
    BETA_CHAIN_TOKEN_SEQ,
    CLASS1_TOKEN_SEQ,
    CLASS2_TOKEN_SEQ,
    MUTANT_TOKEN_SEQ,
)

token_replacement_patterns = {
    # 'MHC' doesn't mean anything by itself since we're always
    # parsing MHC nomenclature here
    ("major", "histocompatibility", "complex"): None,
    ("mhc",): None,
    (
        "major",
        "histocompatibility",
        "antigen",
    ): None,
    (
        "histocompatibility",
        "antigen",
    ): None,
    (
        "leukocyte",
        "antigen",
    ): None,
    # a lot of the HLA sequence s
    ("fragment",): None,
    ("exons", "1-3"): None,
    ("exons", "1-2"): None,
    ("allele", "undetermined"): None,
    # class I
    ("mhc-i",): CLASS1_TOKEN_SEQ,
    ("mhc-1",): CLASS1_TOKEN_SEQ,
    ("class-i",): CLASS1_TOKEN_SEQ,
    ("class", "1"): CLASS1_TOKEN_SEQ,
    ("class", "i"): CLASS1_TOKEN_SEQ,
    ("hla-class", "i"): CLASS1_TOKEN_SEQ,
    ("hla-class", "1"): CLASS1_TOKEN_SEQ,
    # class II
    ("mhc-ii",): CLASS2_TOKEN_SEQ,
    ("mhc-2",): CLASS2_TOKEN_SEQ,
    ("class-ii",): CLASS2_TOKEN_SEQ,
    ("class", "2"): CLASS2_TOKEN_SEQ,
    ("class", "ii"): CLASS2_TOKEN_SEQ,
    ("hla-class", "ii"): CLASS2_TOKEN_SEQ,
    ("hla-class", "2"): CLASS2_TOKEN_SEQ,
    # Class II chains
    ("alpha", "chain"): ALPHA_CHAIN_TOKEN_SEQ,
    ("alpha-chain",): ALPHA_CHAIN_TOKEN_SEQ,
    ("beta", "chain"): BETA_CHAIN_TOKEN_SEQ,
    ("beta-chain",): BETA_CHAIN_TOKEN_SEQ,
    # 'mutant', 'mutation', 'mutations', are synonyms
    ("mutation",): MUTANT_TOKEN_SEQ,
    ("mutations",): MUTANT_TOKEN_SEQ,
}

replacement_pattern_lengths = {len(key) for key in token_replacement_patterns}

# sorted in descending order
sorted_replacement_pattern_lengths = sorted(replacement_pattern_lengths, reverse=True)


def apply_token_substitutions(tokens, length):
    """
    Replace all subsequences of `length` token_strings with any replacements
    found in the dictionary above.
    """
    result_tokens = []
    ignored_tokens = []
    i = 0
    n_tokens = len(tokens)
    while i < n_tokens:
        found_in_dict = False
        if i < n_tokens - length + 1:
            key = tuple([tokens[j].seq for j in range(i, i + length)])
            if key in token_replacement_patterns:
                found_in_dict = True
                new_token_seq = token_replacement_patterns[key]
                if new_token_seq is None:
                    ignored_tokens.extend(key)
                else:
                    new_token = Token(
                        seq=new_token_seq,
                        raw_string=" ".join([tokens[j].raw_string for j in range(i, i + length)]),
                    )
                    result_tokens.append(new_token)

        if found_in_dict:
            i += length
        else:
            result_tokens.append(tokens[i])
            i += 1
    return (
        result_tokens,
        ignored_tokens,
    )


def simplify_tokens(tokens):
    """
    Combine token pairs like ['alpha', 'chain'] into 'alpha-chain' and
    combine the corresponding raw string parts separated by a space.

    Also normalize token sequences like ['mhc' 'class' 'i'] into canonical
    tokens like 'class-1'.
    """
    ignored_tokens = []
    for length in sorted_replacement_pattern_lengths:
        tokens, curr_ignored_tokens = apply_token_substitutions(tokens, length)
        ignored_tokens.extend(curr_ignored_tokens)
    return tokens, ignored_tokens



import pytest

from mhcgnomes import Allele, Gene, Species, parse

from .common import eq_


def test_macaque_allele_B_08_02():
    allele_name = "Mamu-B*082:02"
    eq_(parse(allele_name).to_string(), "Mamu-B*082:02")
    eq_(parse(allele_name).compact_string(), "B08202")


def test_macaque_allele_B_007_02():
    # expect 3rd zero in the family "007" to be trimmed in the normalized form
    # of this allele
    allele_name = "Mamu-B*007:02"
    eq_(parse(allele_name).to_string(), "Mamu-B*007:02")
    eq_(parse(allele_name).compact_string(), "B00702")


def test_gelada_species_Thge():
    expected = Species.get("Thge")
    assert expected is not None
    eq_(parse("Thge"), expected)


def test_gelada_gene_Thge_DQA1():
    expected = Gene.get("Thge", "DQA1")
    assert expected is not None
    eq_(parse("Thge-DQA1"), expected)


def test_gelada_allele_Thge_DQA1_25_01():
    expected = Allele.get("Thge", "DQA1", "25", "01")
    assert expected is not None
    eq_(parse("Thge-DQA1*25:01"), expected)


def test_monkey_species_Aoni():
    expected = Species.get("Aoni")
    assert expected is not None
    eq_(parse("Aoni"), expected)


def test_monkey_gene_Aoni_DRB3():
    expected = Gene.get("Aoni", "DRB3")
    assert expected is not None
    eq_(parse("Aoni-DRB3"), expected)


def test_monkey_allele_Aoni_DRB3_06_01():
    expected = Allele.get("Aoni", "DRB3", "06", "01")
    assert expected is not None
    eq_(parse("Aoni-DRB3*06:01"), expected)


def test_Mamu_A7_allele_without_seps():
    s = "Mamu-A70103"
    eq_(parse(s).to_string(), "Mamu-A7*01:03")


def test_parse_saoe_gene_alias_N():
    result = parse("Saoe-N", raise_on_error=True)
    expected = Gene.get("Saoe", "N1")
    assert expected is not None
    eq_(result, expected)


def test_parse_saoe_allele_alias_N_01_01():
    result = parse("Saoe-N*01:01", raise_on_error=True)
    expected = Allele.get("Saoe", "N1", ("01", "01"))
    assert expected is not None
    eq_(result, expected)


def test_parse_maar_gene_alias_A():
    result = parse("Maar-A", raise_on_error=True)
    expected = Gene.get("Maar", "A1")
    assert expected is not None
    eq_(result, expected)


def test_parse_maar_allele_alias_A_01_01():
    result = parse("Maar-A*01:01", raise_on_error=True)
    expected = Allele.get("Maar", "A1", ("01", "01"))
    assert expected is not None
    eq_(result, expected)


NHP_NOVEL_GENE_EXAMPLES = [
    ("Paha-AG*02:01:01:01", "Paha", "AG", ("02", "01", "01", "01")),
    ("Mamu-K*02:01:01:01", "Mamu", "K", ("02", "01", "01", "01")),
    ("Paha-J*01:01:01:01", "Paha", "J", ("01", "01", "01", "01")),
    ("Paha-K*01:01:01:01", "Paha", "K", ("01", "01", "01", "01")),
    ("Paan-AG*02:01:01:01", "Paan", "AG", ("02", "01", "01", "01")),
    ("Paan-J*01:01:01:01", "Paan", "J", ("01", "01", "01", "01")),
    ("Paan-K*01:01:01:01", "Paan", "K", ("01", "01", "01", "01")),
]


@pytest.mark.parametrize(
    "allele_name,species_prefix,gene_name,_fields",
    NHP_NOVEL_GENE_EXAMPLES,
)
def test_nhp_novel_gene_lookup(allele_name, species_prefix, gene_name, _fields):
    gene = Gene.get(species_prefix, gene_name)
    assert gene is not None
    eq_(gene.name, gene_name)
    eq_(gene.mhc_class, "I")


@pytest.mark.parametrize(
    "allele_name,species_prefix,gene_name,fields",
    NHP_NOVEL_GENE_EXAMPLES,
)
def test_parse_nhp_novel_gene_alleles(allele_name, species_prefix, gene_name, fields):
    allele = parse(allele_name)
    eq_(type(allele), Allele)
    eq_(allele.species.prefix, species_prefix)
    eq_(allele.gene.name, gene_name)
    eq_(allele.allele_fields, fields)
    assert allele.is_class1



from mhcgnomes import AlleleWithoutGene

from .common import eq_


def test_to_string_with_species():
    result = AlleleWithoutGene.get("BoLA", "T2C")
    assert result is not None
    eq_(result.to_string(include_species=True), "BoLA-T2C")


def test_to_string_without_species_has_no_leading_dash():
    result = AlleleWithoutGene.get("BoLA", "T2C")
    assert result is not None
    eq_(result.to_string(include_species=False), "T2C")


def test_compact_string_with_species_flag_stays_compact():
    result = AlleleWithoutGene.get("BoLA", "T2C")
    assert result is not None
    eq_(result.compact_string(include_species=True), "T2C")



# Tests for edge cases that would have caught the bugs fixed in this session
#
# Each test is documented with which bug it would have caught.

from mhcgnomes import (
    Class2Locus,
    Gene,
    Parser,
    Species,
)
from mhcgnomes.common import unique
from mhcgnomes.parsing_helpers import split_on_all_seps
from mhcgnomes.species import guess_class2_chain_type

from .common import eq_

# ============================================================================
# Bug #1: Operator precedence in common.py:20
# Original: if type(xs) is list and len(xs) == 0 or len(xs) == 1:
# This was evaluated as: (type(xs) is list and len(xs) == 0) or (len(xs) == 1)
# So a tuple of length 1 would incorrectly return True for the second part.
# ============================================================================


def test_unique_tuple_length_one():
    """
    A tuple with length 1 should be processed by unique(), not returned as-is.
    Before fix: unique((1,)) would return (1,) unchanged due to operator precedence bug.
    After fix: unique((1,)) correctly processes it (and returns the same since it's unique).
    """
    result = unique((1,))
    # The result should still be [1] after processing, not the original tuple
    assert result == [1], f"Expected [1] but got {result}"


def test_unique_tuple_with_duplicates():
    """
    A tuple with duplicates should have duplicates removed.
    """
    result = unique((1, 2, 2, 3))
    eq_(result, [1, 2, 3])


def test_unique_list_length_zero():
    """
    An empty list should be returned unchanged (optimization).
    """
    empty_list = []
    result = unique(empty_list)
    assert result is empty_list


def test_unique_list_length_one():
    """
    A single-element list should be returned unchanged (optimization).
    """
    single_list = [1]
    result = unique(single_list)
    assert result is single_list


# ============================================================================
# Bug #2 & #7: Missing bounds check and mutable default in class2_locus.py
# Original: while s1[-1].isdigit(): s1 = s1[:-1]
# This crashes with IndexError if the string consists entirely of digits.
# ============================================================================


def test_endswith_ignore_digits_all_digits():
    """
    endswith_ignore_digits should handle strings that are all digits without crashing.
    Before fix: Class2Locus.endswith_ignore_digits("123", "a") would raise IndexError.
    """
    result = Class2Locus.endswith_ignore_digits("123", "a")
    eq_(result, False)


def test_endswith_ignore_digits_empty_after_strip():
    """
    endswith_ignore_digits should handle strings that become empty after stripping digits.
    """
    result = Class2Locus.endswith_ignore_digits("999", "")
    eq_(result, True)  # Empty string ends with empty string


def test_endswith_ignore_digits_normal():
    """
    Normal case: "DRA1" should match "A" after stripping trailing digit.
    """
    result = Class2Locus.endswith_ignore_digits("dra1", "a")
    eq_(result, True)


def test_class2_locus_default_genes():
    """
    Class2Locus should handle default genes argument correctly.
    Before fix: genes=[] as default was a mutable default argument bug.
    """
    species = Species.get("HLA")
    locus1 = Class2Locus(species=species, name="DR")
    locus2 = Class2Locus(species=species, name="DQ")
    # Ensure they have separate gene lists
    eq_(locus1.genes, [])
    eq_(locus2.genes, [])


# ============================================================================
# Bug #3: Missing bounds check in species.py:438 (guess_class2_chain_type)
# Similar to bug #2 - while loop without bounds check on digit stripping.
# ============================================================================


def test_guess_class2_chain_type_all_digits():
    """
    guess_class2_chain_type should handle gene names that are all digits.
    Before fix: guess_class2_chain_type("123") would raise IndexError.
    """
    result = guess_class2_chain_type("123")
    # Default to beta when can't determine
    eq_(result, "beta")


def test_guess_class2_chain_type_ends_with_like_then_digits():
    """
    Handle gene names like "123like456" that become all digits after stripping "like".
    """
    result = guess_class2_chain_type("like123")
    eq_(result, "beta")


def test_guess_class2_chain_type_alpha():
    """
    Normal case: gene names ending in 'A' (after digit stripping) are alpha chains.
    """
    result = guess_class2_chain_type("DRA1")
    eq_(result, "alpha")


def test_guess_class2_chain_type_beta():
    """
    Normal case: gene names ending in 'B' (after digit stripping) are beta chains.
    """
    result = guess_class2_chain_type("DRB1")
    eq_(result, "beta")


# ============================================================================
# Bug #4: Index before check in parser.py:837 (parse_and_apply_mutations)
# The function accessed mutation_tokens[-1] before checking if list was empty.
# ============================================================================


def test_parse_and_apply_mutations_empty_tokens():
    """
    parse_and_apply_mutations should handle empty mutation tokens gracefully.
    Before fix: would crash with IndexError when mutation_tokens was empty.
    """
    parser = Parser()
    gene = Gene.get("HLA", "A")
    result = parser.parse_and_apply_mutations(gene, [])
    eq_(result, None)


# ============================================================================
# Bug #5: Uninitialized variable in parsing_helpers.py:51 (split_on_all_seps)
# If seps="" (empty string), the for loop never executes and 'parts' is undefined.
# ============================================================================


def test_split_on_all_seps_empty_seps():
    """
    split_on_all_seps should handle empty separator string without NameError.
    Before fix: split_on_all_seps("test", "") would raise NameError.
    """
    result = split_on_all_seps("test", "")
    eq_(result, ["test"])


def test_split_on_all_seps_normal():
    """
    Normal case: split on multiple separators.
    """
    result = split_on_all_seps("02_01:01", "_:")
    eq_(result, ["02", "01", "01"])


def test_split_on_all_seps_single_sep():
    """
    Single separator.
    """
    result = split_on_all_seps("a:b:c", ":")
    eq_(result, ["a", "b", "c"])


# ============================================================================
# Bug #6: Regex warnings (SyntaxWarning from invalid escape sequences)
# These tests ensure the regex patterns still work correctly after conversion
# to raw strings.
# ============================================================================


def test_mutation_regex_still_works():
    """
    Mutation regex should work correctly after conversion to raw string.
    """
    from mhcgnomes.mutation import Mutation

    result = Mutation.parse("G86Y")
    assert result is not None
    eq_(result.aa_original, "G")
    eq_(result.pos, 86)
    eq_(result.aa_mutant, "Y")


def test_standard_format_regex_still_works():
    """
    Standard allele format regex should work correctly after conversion to raw strings.
    """
    from mhcgnomes.standard_format import parse_standard_allele_format

    result = parse_standard_allele_format("HLA-A*02:01")
    assert result is not None
    eq_(result.gene.name, "A")
    eq_(result.allele_fields, ("02", "01"))


def test_compact_gene_and_allele_regex_still_works():
    """
    The compact gene and allele regex in Parser should work correctly.
    """
    parser = Parser()
    match = parser.compact_gene_and_allele_regex.fullmatch("A0201")
    assert match is not None
    groups = match.groups()
    eq_(groups[0], "A")
    eq_(groups[1], "0201")


# ============================================================================
# Additional edge case tests that provide good coverage
# ============================================================================


def test_unique_with_generator():
    """
    unique() should work with generators (non-list iterables).
    """
    result = unique(x for x in [1, 2, 2, 3])
    eq_(result, [1, 2, 3])


def test_parse_mutation_with_digits():
    """
    Parse mutation with multi-digit position.
    """
    from mhcgnomes.mutation import Mutation

    result = Mutation.parse("A123B")
    assert result is not None
    eq_(result.pos, 123)


def test_class2_locus_alpha_beta_chain_genes_empty():
    """
    Class2Locus with no genes should return empty lists for chain genes.
    """
    species = Species.get("HLA")
    locus = Class2Locus(species=species, name="DR", genes=[])
    eq_(locus.alpha_chain_genes, [])
    eq_(locus.beta_chain_genes, [])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .result_with_mhc_class import ResultWithMhcClass


class ResultWithGene(ResultWithMhcClass):
    """
    Common base class for any result object which has a species field.

    Useful for sharing helper methods that rely on the 'species' field.
    """

    def __init__(self, gene, raw_string=None):
        ResultWithMhcClass.__init__(
            self, species=gene.species, mhc_class=gene.mhc_class, raw_string=raw_string
        )
        self.gene = gene

    @property
    def has_gene(self):
        return True

    @property
    def gene_name(self):
        return self.gene.name

    @property
    def is_class2_alpha(self):
        return (
            self.is_class2
            and self.species.class2_gene_name_to_chain_type[self.gene_name] == "alpha"
        )

    @property
    def is_class2_beta(self):
        return (
            self.is_class2 and self.species.class2_gene_name_to_chain_type[self.gene_name] == "beta"
        )



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections.abc import Sequence
from typing import Union

from .allele import Allele
from .result_with_multiple_alleles import ResultWithMultipleAlleles
from .species import Species


class Supertype(ResultWithMultipleAlleles):
    """
    Represents an HLA supertype (functional peptide-binding group).

    A Supertype represents a group of HLA alleles that share similar peptide
    binding properties based on their B and F pocket chemistry. Unlike serotypes
    (which are based on antibody recognition), supertypes are defined by
    functional similarity in the peptides they can bind.

    The nine major HLA class I supertypes (A01, A02, A03, A24, B07, B08, B27,
    B44, B58, B62) were defined by Sidney, Sette et al. based on analysis of
    peptide-binding pockets.

    Note that supertype membership may differ from serotype classification.
    For example, A*68:02 is serologically A68 but belongs to the A02 supertype
    because it binds similar peptides to A*02:01.

    Parameters
    ----------
    species : Species
        The species this supertype belongs to (typically HLA/human).
    name : str
        The supertype name (e.g., "A01", "A02", "B07").
    alleles : Sequence[Allele]
        The alleles that belong to this supertype.
    representative : Allele, optional
        The representative/prototypic allele for this supertype.
    raw_string : str, optional
        The original unparsed string.

    Attributes
    ----------
    species : Species
        The species this supertype belongs to.
    name : str
        The supertype name.
    alleles : Sequence[Allele]
        The alleles in this supertype.
    representative : Allele or None
        The representative allele for this supertype.

    Examples
    --------
    >>> from mhcgnomes import parse
    >>> st = parse("A2 supertype")
    >>> st.name
    'A02'
    >>> len(st.alleles) > 0
    True

    References
    ----------
    Sidney J, Peters B, Frahm N, Brander C, Sette A. "HLA class I supertypes:
    a revised and updated classification." BMC Immunol. 2008;9:1.
    https://pmc.ncbi.nlm.nih.gov/articles/PMC2245908/
    """

    def __init__(
        self,
        species: Species,
        name: str,
        alleles: Sequence[Allele],
        representative: Union[Allele, None] = None,
        raw_string: Union[str, None] = None,
    ):
        ResultWithMultipleAlleles.__init__(
            self, species=species, name=name, alleles=alleles, raw_string=raw_string
        )
        self.representative = representative

    def to_string(self, include_species=True, use_old_species_prefix=False):
        if include_species:
            prefix = self.species.to_string(use_old_species_prefix=use_old_species_prefix)
            return f"{prefix} {self.name} supertype"
        else:
            return f"{self.name} supertype"

    def compact_string(self, include_species=False, use_old_species_prefix=False):
        if include_species:
            prefix = self.species.to_string(use_old_species_prefix=use_old_species_prefix)
            return f"{prefix}-{self.name}"
        else:
            return self.name

    def to_record(self):
        d = self.species.to_record()
        d["supertype"] = self.name
        d["supertype_string"] = self.to_string()
        if self.representative:
            d["representative_allele"] = self.representative.to_string()
        return d



from mhcgnomes import Allele, Species, parse


def test_Bofr_species():
    species = Species.get("Bofr")
    assert species is not None
    assert len(species.gene_names) > 0


def test_Bofr_DQB1_010_11():
    result = parse("Bofr-DQB1*010:11")
    assert result is not None
    assert type(result) is Allele
    assert result.species.prefix == "Bofr"
    assert result.gene.name == "DQB1"
    assert result.allele_fields[0] == "010"
    assert result.allele_fields[1] == "11"



from mhcgnomes import Species, parse

from .common import eq_


def test_HLA_A2_in_species_serotype_dictionary():
    human = Species.get("HLA")
    assert human is not None
    assert "A2" in human.serotypes


def test_parse_HLA_A2_serotype():
    result = parse("HLA-A2")
    assert result is not None
    eq_(result.name, "A2")
    eq_(result.species.prefix, "HLA")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections.abc import Iterable
from functools import wraps


def unique(xs: Iterable):
    """
    Return iterable at most as long as the input, containing only its
    unique elements.
    """
    if type(xs) is list and (len(xs) == 0 or len(xs) == 1):
        return xs
    result = []
    unique_set = set()
    for xi in xs:
        if xi in unique_set:
            continue
        result.append(xi)
        unique_set.add(xi)
    return result


def arg_to_cache_key(x):
    if x is None:
        return None

    t = type(x)

    if t is int or t is str or t is bool or t is float or t is type:
        return x

    if t is list or t is tuple:
        if len(x) == 0:
            return ()
        elif len(x) == 1:
            return (arg_to_cache_key(x[0]),)
        else:
            value = tuple([arg_to_cache_key(xi) for xi in x])
    elif t is dict:
        if len(x) == 0:
            value = ()
        elif len(x) == 1:
            ((k, v),) = x.items()
            value = (arg_to_cache_key(k), arg_to_cache_key(v))
        else:
            value = tuple([(arg_to_cache_key(k), arg_to_cache_key(v)) for (k, v) in x.items()])
    else:
        value = x
    return (t.__name__, value)


def cache(fn):
    """
    Memoization function which tries to freeze non-hashable types like
    dict and list, which depends on functions being called not modifying their
    arguments.
    """
    cache_dict = {}

    @wraps(fn)
    def cached_fn(*args, **kwargs):
        args_key = arg_to_cache_key(args)
        if not kwargs:
            kwargs_key = ()
        else:
            kwargs_key = arg_to_cache_key(kwargs)
        key = (args_key, kwargs_key)
        if key not in cache_dict:
            result = fn(*args, **kwargs)
            cache_dict[key] = result
        return cache_dict[key]

    return cached_fn


def normalize_string(name, _cache=None):
    """
    Return uppercase string without any surrounding whitespace and
    without any characters such as '-', '_' ':' or "'"
    """
    if _cache is None:
        _cache = {}
    if name is None:
        return None

    if name in _cache:
        return _cache[name]

    t = type(name)
    if t is float or t is int:
        result = str(name)
    elif t is not str:
        result = name
    else:
        result = (
            name.replace("-", "").replace("_", "").replace("'", "").replace(":", "").strip().upper()
        )
    _cache[name] = result
    return result



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .mhc_class_helpers import is_class1, is_class2
from .result_with_species import ResultWithSpecies


class ResultWithMhcClass(ResultWithSpecies):
    """
    Common base class for any result object which has a species field.

    Useful for sharing helper methods that rely on the 'species' field.
    """

    def __init__(self, species, mhc_class, raw_string=None):
        ResultWithSpecies.__init__(self, species=species, raw_string=raw_string)
        self.mhc_class = mhc_class

    @property
    def has_mhc_class(self):
        return True

    @property
    def is_class1(self):
        return is_class1(self.mhc_class)

    @property
    def is_class2(self):
        return is_class2(self.mhc_class)



from mhcgnomes import Allele, Gene, Haplotype, Species, parse

from .common import eq_


def test_parse_BF2_gene():
    result = parse("BF2")
    expected = Gene.get("chicken", "BF2")
    assert expected is not None
    eq_(result, expected)


def test_parse_BF2_15_01():
    result = parse("BF2*15:01")
    chicken = Species.get("chicken")
    expected = Allele.get(chicken, "BF2", "15", "01")
    eq_(result, expected)


def test_parse_BF2_1501():
    result = parse("BF2*1501")
    expected = Allele.get(Species.get("chicken"), "BF2", "15", "01")
    eq_(result, expected)


def test_chicken_haplotype_B12():
    result = parse("B12", default_species="Gaga")
    print(result)
    eq_(type(result), Haplotype)
    eq_(result.name, "B12")
    assert result.is_chicken


def test_chicken_haplotype_B19_class_II():
    result = parse("B19 Class II")
    eq_(type(result), Haplotype)
    eq_(result.name, "B19")
    assert result.is_chicken
    assert result.is_class2


def test_chicken_haplotype_BF19_class_I():
    result = parse("BF19 Class I")
    eq_(type(result), Haplotype)
    eq_(result.name, "BF19")
    assert result.is_chicken
    assert result.is_class1


def test_chicken_YF1w_7_1():
    result = parse("YF1w*7.1")
    eq_(type(result), Allele)
    eq_(result.name, "7.1")
    eq_(result.gene_name, "YF1")
    assert result.is_chicken
    assert result.is_class1


def test_YF1w_in_gene_aliases():
    species = Species.get("chicken")
    gene_names_and_aliases = species.gene_names_and_aliases
    assert "YF1w" in gene_names_and_aliases, gene_names_and_aliases



from mhcgnomes import Allele, Gene, Species, parse

from .common import eq_


def test_parse_existing_snake_species_sica():
    expected = Species.get("Sica")
    assert expected is not None
    eq_(parse("Sica"), expected)


def test_parse_existing_snake_gene_sica_daa():
    expected = Gene.get("Sica", "DAA")
    assert expected is not None
    eq_(parse("Sica-DAA"), expected)


def test_parse_existing_snake_allele_sica_daa_01_01():
    expected = Allele.get("Sica", "DAA", "01", "01")
    assert expected is not None
    eq_(parse("Sica-DAA*01:01"), expected)


def test_parse_existing_frog_species_xela():
    expected = Species.get("Xela")
    assert expected is not None
    eq_(parse("Xela"), expected)


def test_parse_existing_frog_gene_xela_uaa():
    expected = Gene.get("Xela", "UAA")
    assert expected is not None
    eq_(parse("Xela-UAA"), expected)


def test_parse_existing_frog_allele_xela_uaa_01_01():
    expected = Allele.get("Xela", "UAA", "01", "01")
    assert expected is not None
    eq_(parse("Xela-UAA*01:01"), expected)


def test_parse_pilot_frog_species_xetr():
    expected = Species.get("Xetr")
    assert expected is not None
    eq_(parse("Xetr"), expected)
    eq_(Species.get("Xenopus tropicalis"), expected)


def test_parse_pilot_lizard_species_anca():
    expected = Species.get("Anca")
    assert expected is not None
    eq_(parse("Anca"), expected)
    eq_(Species.get("Anolis carolinensis"), expected)


def test_parse_pilot_lizard_species_ansa():
    expected = Species.get("Ansa")
    assert expected is not None
    eq_(parse("Ansa"), expected)
    eq_(Species.get("Anolis sagrei"), expected)



import argparse
import sys
import xml.etree.ElementTree as ET
from collections import OrderedDict, defaultdict

import pandas as pd
import yaml

parser = argparse.ArgumentParser(
    prog="combine_allele_aliases.py",
    description="Combine databases with manually curated allele aliases",
)

parser.add_argument("--yaml-input-file", "-y", nargs="+")
parser.add_argument("--xml-input-file", "-x", nargs="+")
parser.add_argument("--allele-history-input-file", "-a", nargs="+")
parser.add_argument("--csv-input-file", "-s", nargs="+")
parser.add_argument("--output", "-o", required=True)


def valid_name(name):
    if "(" in name or ")" in name:
        return False
    return not name.replace(":", "").isdigit()


def sufficiently_different_name(old_name, new_name):
    if old_name == new_name:
        return False
    old_name_without_seps = old_name.replace(":", "")
    new_name_without_seps = new_name.replace(":", "")
    if old_name_without_seps == new_name_without_seps:
        return False
    return not (old_name.count(":") > 0 and old_name_without_seps in new_name)


class MappingAccumulator:
    def __init__(self):
        self.species_to_old_to_new = defaultdict(OrderedDict)
        self.ambiguous_names = defaultdict(set)

    def update(self, species, old_name, new_name):
        old_name = old_name.strip()
        if not valid_name(old_name):
            return False
        if old_name in self.ambiguous_names[species]:
            return False
        new_name = new_name.strip()
        if not valid_name(new_name):
            return False
        if not sufficiently_different_name(old_name, new_name):
            return False
        if old_name in self.species_to_old_to_new[species]:
            existing_value = self.species_to_old_to_new[species][old_name]
            if new_name == existing_value or existing_value.startswith(new_name):
                return False
            elif new_name.startswith(existing_value):
                # if there's a version of the new allele with more
                # fields then keep that instead
                self.species_to_old_to_new[species][old_name] = new_name
                return True
            else:
                del self.species_to_old_to_new[species][old_name]
                self.ambiguous_names[species].add(old_name)
                return False
        else:
            self.species_to_old_to_new[species][old_name] = new_name
            return True

    def merge_dictionary(self, d):
        for species, species_dict in d.items():
            for old_name, new_name in species_dict.items():
                self.update(species, old_name, new_name)


def main(args_list):
    args = parser.parse_args(args_list)
    accum = MappingAccumulator()

    if args.xml_input_file:
        for xml_file in args.xml_input_file:
            print(f"Loading XML source {xml_file}")
            tree = ET.parse(xml_file)
            root = tree.getroot()
            entries = root.find("entries")
            for entry in entries:
                name_obj = entry.find("name")
                if name_obj is None:
                    continue
                name = name_obj.text
                if "-" not in name:
                    raise ValueError(f"Missing species in {name}")
                parts = name.split("-")
                species = parts[0]
                name_without_species = "-".join(parts[1:])
                nomenclature_obj = entry.find("nomenclature")
                if not nomenclature_obj:
                    continue
                for old_name_obj in nomenclature_obj:
                    old_names = old_name_obj.text
                    for old_name in old_names.split(","):
                        accum.update(species, old_name, name_without_species)

    if args.yaml_input_file:
        for yaml_filename in args.yaml_input_file:
            print(f"Loading YAML source {yaml_filename}")
            with open(yaml_filename) as yaml_file:
                d = yaml.safe_load(yaml_file.read())
                accum.merge_dictionary(d)

    if args.allele_history_input_file:
        for filename in args.allele_history_input_file:
            print(f"Loading allele history source {filename}")
            with open(filename) as f:
                for line in f:
                    line = line.strip()
                    if not line or line.startswith("#"):
                        continue
                    parts = line.split(",")
                    most_recent_name = parts[1]
                    if most_recent_name == "NA":
                        continue

                    old_names = [p for p in parts[2:] if p and p != "NA"]
                    different_old_names = [
                        name
                        for name in old_names
                        if name != most_recent_name
                        and valid_name(name)
                        and sufficiently_different_name(name, most_recent_name)
                    ]
                    for old_name in different_old_names:
                        if accum.update("HLA", old_name, most_recent_name):
                            print(f"\t{old_name}=>{most_recent_name}")

    if args.csv_input_file:
        for filename in args.csv_input_file:
            print(f"Loading CSV source {filename}")
            df = pd.read_csv(filename, comment="#")
            assert "Allele" in df.columns
            assert "Description" in df.columns
            for old_allele, desc in zip(df.Allele, df.Description):
                new_allele = None
                for sep in ["=", "renamed", "identical to", "changed to", "Renamed"]:
                    if sep in desc:
                        parts = desc.split(sep)
                        part = parts[1]
                        new_allele = part.split()[0]
                        break
                if new_allele:
                    accum.update("HLA", old_allele, new_allele)
                else:
                    print(f"\tSkipping {old_allele}: {desc}")
    #  sanity check to make sure no "new name" also has an "old name" entry
    changed = True
    species_to_old_to_new = accum.species_to_old_to_new
    while changed:
        print("\n")
        changed = False
        updated = species_to_old_to_new.copy()
        for species, species_dict in species_to_old_to_new.items():
            for old, new in species_dict.items():
                if "-" in new:
                    if new.startswith(f"{species}-"):
                        new_without_species = new[len(species) + 1 :]
                        print(f"Removing species prefix from {new} = {new_without_species}")
                        del updated[species][old]
                        updated[species][old] = new_without_species
                        changed = True
                elif new in species_dict:
                    newer = species_dict[new]
                    print(f"\t Updating {species} {old} => {new} => {newer}")
                    updated[species][old] = newer
                    changed = True

        species_to_old_to_new = updated

    print(
        "Writing %d alias mappings across %d species to %s"
        % (
            sum([len(species_dict) for species_dict in species_to_old_to_new.values()]),
            len(species_to_old_to_new),
            args.output,
        )
    )
    with open(args.output, "w") as f:
        for species, species_dict in sorted(species_to_old_to_new.items()):
            f.write(f"{species}:\n")
            for old_name, new_name in sorted(species_dict.items()):
                f.write(f"  {old_name}: {new_name}\n")


if __name__ == "__main__":
    main(sys.argv[1:])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import OrderedDict
from typing import Union

from .allele import Allele
from .allele_without_gene import AlleleWithoutGene
from .gene import Gene
from .result_with_mhc_class import ResultWithMhcClass


class Pair(ResultWithMhcClass):
    """
    Represents an alpha/beta pair of MHC Class II chains.

    MHC Class II molecules are heterodimers consisting of an alpha chain
    and a beta chain. This class represents such pairs, commonly seen in
    notation like "HLA-DRA*01:01/DRB1*01:01".

    Parameters
    ----------
    alpha : Allele, AlleleWithoutGene, or Gene
        The alpha chain component.
    beta : Allele, AlleleWithoutGene, or Gene
        The beta chain component.
    mhc_class : str, default "II"
        The MHC class (typically "II" for pairs).
    raw_string : str, optional
        The original unparsed string.

    Attributes
    ----------
    alpha : Allele, AlleleWithoutGene, or Gene
        The alpha chain component.
    beta : Allele, AlleleWithoutGene, or Gene
        The beta chain component.
    species : Species
        The species (inherited from alpha chain).
    mhc_class : str
        The MHC class.

    Examples
    --------
    >>> from mhcgnomes import parse
    >>> pair = parse("HLA-DRA*01:01/DRB1*01:01")
    >>> pair.alpha.gene_name
    'DRA'
    >>> pair.beta.gene_name
    'DRB1'
    >>> pair.to_string()
    'HLA-DRA*01:01/DRB1*01:01'
    """

    def __init__(
        self,
        alpha: Union[Allele, AlleleWithoutGene, Gene],
        beta: Union[Allele, AlleleWithoutGene, Gene],
        mhc_class="II",
        raw_string: Union[str, None] = None,
    ):
        ResultWithMhcClass.__init__(
            self, species=alpha.species, mhc_class=mhc_class, raw_string=raw_string
        )
        self.alpha = alpha
        self.beta = beta

    @classmethod
    def hash_field_names(cls):
        return ("alpha", "beta")

    @classmethod
    def eq_field_names(cls):
        return ("alpha", "beta")

    @classmethod
    def get(cls, alpha, beta, raw_string=None):
        if alpha is None:
            return None
        if beta is None:
            return None
        if not isinstance(alpha, (Allele, Gene, AlleleWithoutGene)):
            return None
        if not isinstance(beta, (Allele, Gene, AlleleWithoutGene)):
            return None
        if alpha.mhc_class == "other" and beta.mhc_class != "other":
            mhc_class = beta.mhc_class
        elif (
            beta.mhc_class == "other" and alpha.mhc_class != "other"
        ) or alpha.mhc_class == beta.mhc_class:
            mhc_class = alpha.mhc_class
        else:
            mhc_class = "II"

        return cls(alpha=alpha, beta=beta, mhc_class=mhc_class, raw_string=raw_string)

    def to_string(self, include_species=True, use_old_species_prefix=False):
        return "{}/{}".format(
            self.alpha.to_string(
                include_species=include_species, use_old_species_prefix=use_old_species_prefix
            ),
            self.beta.to_string(include_species=False),
        )

    def compact_string(self, include_species=False, use_old_species_prefix=False):
        return "{}-{}".format(
            self.alpha.compact_string(
                include_species=include_species, use_old_species_prefix=use_old_species_prefix
            ),
            self.beta.compact_string(include_species=False),
        )

    @property
    def gene_name(self):
        return f"{self.alpha.gene_name}/{self.beta.gene_name}"

    def to_record(self):
        # return a dictionary that has the same elements as Gene.to_dict()
        # along with "allele"
        return OrderedDict(
            [
                ("gene", self.gene_name),
                ("mhc_class", self.mhc_class),
                ("is_mutant", False),
                ("allele", self.to_string()),
            ]
        )

    def restrict_allele_fields(
        self, num_fields: int, drop_annotations: bool = False, drop_mutations: bool = False
    ):
        alpha = self.alpha.restrict_allele_fields(
            num_fields=num_fields, drop_annotations=drop_annotations, drop_mutations=drop_mutations
        )
        beta = self.beta.restrict_allele_fields(
            num_fields=num_fields, drop_annotations=drop_annotations, drop_mutations=drop_mutations
        )
        if alpha != self.alpha or beta != self.beta:
            return Pair.get(alpha=alpha, beta=beta, raw_string=self.raw_string)
        else:
            return self

    @property
    def annotation_null(self):
        return self.alpha.annotation_null or self.beta.annotation_null

    @property
    def annotation_cystosolic(self):
        return self.alpha.annotation_cystosolic or self.beta.annotation_cystosolic

    @property
    def annotation_secreted(self):
        return self.alpha.annotation_secreted or self.beta.annotation_secreted

    @property
    def annotation_questionable(self):
        return self.alpha.annotation_questionable or self.beta.annotation_questionable

    @property
    def annotation_low_expression(self):
        return self.alpha.annotation_low_expression or self.beta.annotation_low_expression

    @property
    def annotation_aberrant_expression(self):
        return self.alpha.annotation_aberrant_expression or self.beta.annotation_aberrant_expression

    @property
    def annotation_group(self):
        return self.alpha.annotation_group or self.beta.annotation_group

    @property
    def annotation_pseudogene(self):
        return self.alpha.annotation_pseudogene or self.beta.annotation_pseudogene

    @property
    def annotation_splice_variant(self):
        return self.alpha.annotation_splice_variant or self.beta.annotation_splice_variant


default_human_alpha_chains = {
    # The DR alpha chain is effectively constant across the population
    "DRB": Allele.get("HLA", "DRA", "01", "01"),
    # Most common alpha chain for DP is DPA*01:03 but we really
    # need to change this logic to use a lookup table of pairwise
    # frequencies for inferring the alpha-beta pairing
    "DPB": Allele.get("HLA", "DPA1", "01", "03"),
    # Most common DQ alpha (according to wikipedia) is DQA1*01:02
    # but like DPA we should use pair frequencies in the future
    "DQB": Allele.get("HLA", "DQA1", "01", "02"),
}


def infer_class2_alpha_chain(beta):
    """
    Given an Allele for a Class II beta chain, returns the alpha/beta pair with
    a population invariant alpha chain (if one is known)
    """
    if isinstance(beta, Pair):
        return beta

    if not isinstance(beta, Allele):
        return beta

    if not beta.is_class2:
        return beta

    if beta.species_prefix != "HLA":
        return beta

    locus = beta.gene_name[:3]
    if locus not in default_human_alpha_chains:
        return beta

    alpha = default_human_alpha_chains.get(locus)

    if alpha is None:
        return beta

    return Pair.get(alpha, beta)



from mhcgnomes import Allele, parse
from mhcgnomes.allele_annotations import (
    parse_annotations_from_allele_fields,
)

from .common import eq_


def test_questionable_A_01_281Q():
    result = parse("A*01:281Q")
    eq_(result, Allele.get("HLA", "A", "01", "281", annotation="Q"))
    eq_(result.annotations, ("Q",))
    assert result.annotation_questionable


def test_questionable_A01281Q():
    result = parse("A01281Q")
    eq_(result, Allele.get("HLA", "A", "01", "281", annotation="Q"))
    eq_(result.annotations, ("Q",))
    assert result.annotation_questionable


def test_null_E_01_08_01N():
    result = parse("E*01:08:01N")
    eq_(result, Allele.get("HLA", "E", "01", "08", "01", annotation="N"))
    eq_(result.annotations, ("N",))
    assert result.annotation_null


def test_null_E010801N():
    result = parse("E010801N")
    eq_(result, Allele.get("HLA", "E", "01", "08", "01", annotation="N"))
    eq_(result.annotations, ("N",))
    assert result.annotation_null


def test_parse_functional_annotations_from_allele_fields_E010801N():
    results = parse_annotations_from_allele_fields("010801N")
    assert len(results) == 3
    eq_(results[0], ("010801",))
    eq_(results[1], ())
    eq_(results[2], ("N",))


def test_parse_workshop_annotation_from_allele_fields_W113G():
    results = parse_annotations_from_allele_fields("W113G")
    assert len(results) == 3
    eq_(results[0], ("113",))
    eq_(results[1], ("W",))
    eq_(results[2], ("G",))


def test_secreted_B_44_02_01_02S():
    result = parse("B*44:02:01:02S")
    eq_(result, Allele.get("HLA", "B", "44", "02", "01", "02", annotation="S"))
    eq_(result.annotations, ("S",))
    assert result.annotation_secreted


def test_secreted_B_44020102S():
    result = parse("B*44020102S")
    eq_(result, Allele.get("HLA", "B", "44", "02", "01", "02", annotation="S"))
    eq_(result.annotations, ("S",))
    assert result.annotation_secreted


def test_splice_variant_annotation_Sp():
    result = parse("Mamu-B*074:03Sp")
    eq_(result.annotations, ("Sp",))
    assert result.annotation_splice_variant


def test_pseudogene_annotation_Ps():
    result = parse("Caja-B5*01:01Ps")
    eq_(result.annotations, ("Ps",))
    assert result.annotation_pseudogene


def test_Ps_in_gene_name_not_annotation():
    result = parse("Caja-G2Ps*01")
    eq_(result.gene_name, "G2Ps")
    eq_(result.annotations, ())



from mhcgnomes import parse


def test_Anpl_UAA_01():
    # Anpl-UAA*01 occurs 34 times in 2019 IEDB snapshot
    assert parse("Anpl-UAA*01") is not None


def test_B12_class_I():
    # B12 class I occurs 4 times in 2019 IEDB snapshot
    assert parse("B12 class I") is not None


def test_B19_class_I():
    # B19 class I occurs 30 times in 2019 IEDB snapshot
    assert parse("B19 class I") is not None


def test_BF1_0201():
    # BF1*0201 occurs 2 times in 2019 IEDB snapshot
    assert parse("BF1*0201") is not None


def test_BF19_class_II():
    # BF19 class II occurs 1 times in 2019 IEDB snapshot
    assert parse("BF19 class II") is not None


def test_BF2():
    # BF2 occurs 2 times in 2019 IEDB snapshot
    assert parse("BF2") is not None


def test_BF2_0201():
    # BF2*0201 occurs 13 times in 2019 IEDB snapshot
    assert parse("BF2*0201") is not None


def test_BF2_0401():
    # BF2*0401 occurs 33 times in 2019 IEDB snapshot
    assert parse("BF2*0401") is not None


def test_BF2_1201():
    # BF2*1201 occurs 12 times in 2019 IEDB snapshot
    assert parse("BF2*1201") is not None


def test_BF2_1301():
    # BF2*1301 occurs 30 times in 2019 IEDB snapshot
    assert parse("BF2*1301") is not None


def test_BF2_1401():
    # BF2*1401 occurs 9 times in 2019 IEDB snapshot
    assert parse("BF2*1401") is not None


def test_BF2_1501():
    # BF2*1501 occurs 2 times in 2019 IEDB snapshot
    assert parse("BF2*1501") is not None


def test_BF2_2101():
    # BF2*2101 occurs 82 times in 2019 IEDB snapshot
    assert parse("BF2*2101") is not None


def test_BF21_class_II():
    # BF21 class II occurs 4 times in 2019 IEDB snapshot
    assert parse("BF21 class II") is not None


def test_BoLA_1_02101():
    # BoLA-1*02101 occurs 2 times in 2019 IEDB snapshot
    assert parse("BoLA-1*02101") is not None


def test_BoLA_2_00801():
    # BoLA-2*00801 occurs 14 times in 2019 IEDB snapshot
    assert parse("BoLA-2*00801") is not None


def test_BoLA_2_01201():
    # BoLA-2*01201 occurs 60 times in 2019 IEDB snapshot
    assert parse("BoLA-2*01201") is not None


def test_BoLA_2_01801():
    # BoLA-2*01801 occurs 6 times in 2019 IEDB snapshot
    assert parse("BoLA-2*01801") is not None


def test_BoLA_4_02401():
    # BoLA-4*02401 occurs 4 times in 2019 IEDB snapshot
    assert parse("BoLA-4*02401") is not None


def test_BoLA_6_01301():
    # BoLA-6*01301 occurs 92 times in 2019 IEDB snapshot
    assert parse("BoLA-6*01301") is not None


def test_BoLA_6_01302():
    # BoLA-6*01302 occurs 1 times in 2019 IEDB snapshot
    assert parse("BoLA-6*01302") is not None


def test_BoLA_6_04101():
    # BoLA-6*04101 occurs 2 times in 2019 IEDB snapshot
    assert parse("BoLA-6*04101") is not None


def test_BoLA_A11():
    # BoLA-A11 occurs 40 times in 2019 IEDB snapshot
    assert parse("BoLA-A11") is not None


def test_BoLA_DQ():
    # BoLA-DQ occurs 1 times in 2019 IEDB snapshot
    assert parse("BoLA-DQ") is not None


def test_BoLA_DR():
    # BoLA-DR occurs 3 times in 2019 IEDB snapshot
    assert parse("BoLA-DR") is not None


def test_BoLA_DRB3_0101():
    # BoLA-DRB3*0101 occurs 6 times in 2019 IEDB snapshot
    assert parse("BoLA-DRB3*0101") is not None


def test_BoLA_DRB3_0201():
    # BoLA-DRB3*0201 occurs 5 times in 2019 IEDB snapshot
    assert parse("BoLA-DRB3*0201") is not None


def test_BoLA_DRB3_1101():
    # BoLA-DRB3*1101 occurs 5 times in 2019 IEDB snapshot
    assert parse("BoLA-DRB3*1101") is not None


def test_BoLA_DRB3_1201():
    # BoLA-DRB3*1201 occurs 24 times in 2019 IEDB snapshot
    assert parse("BoLA-DRB3*1201") is not None


def test_BoLA_DRB3_2703():
    # BoLA-DRB3*2703 occurs 14 times in 2019 IEDB snapshot
    assert parse("BoLA-DRB3*2703") is not None


def test_BoLA_DRB6():
    # BoLA-DRB6 occurs 37 times in 2019 IEDB snapshot
    assert parse("BoLA-DRB6") is not None


def test_Caja_E():
    # Caja-E occurs 13 times in 2019 IEDB snapshot
    assert parse("Caja-E") is not None


def test_DLA_88_50101():
    # DLA-88*50101 occurs 1963 times in 2019 IEDB snapshot
    assert parse("DLA-88*50101") is not None


def test_DLA_88_50801():
    # DLA-88*50801 occurs 20 times in 2019 IEDB snapshot
    assert parse("DLA-88*50801") is not None


def test_ELA_A1_class_I():
    # ELA-A1 class I occurs 21 times in 2019 IEDB snapshot
    assert parse("ELA-A1 class I") is not None


def test_Eqca_1_00101():
    # Eqca-1*00101 occurs 702 times in 2019 IEDB snapshot
    assert parse("Eqca-1*00101") is not None


def test_Eqca_1_00201():
    # Eqca-1*00201 occurs 80 times in 2019 IEDB snapshot
    assert parse("Eqca-1*00201") is not None


def test_Eqca_16_00101():
    # Eqca-16*00101 occurs 80 times in 2019 IEDB snapshot
    assert parse("Eqca-16*00101") is not None


def test_Eqca_N_00601():
    # Eqca-N*00601 occurs 4 times in 2019 IEDB snapshot
    assert parse("Eqca-N*00601") is not None


def test_Eqca_N_00602():
    # Eqca-N*00602 occurs 21 times in 2019 IEDB snapshot
    assert parse("Eqca-N*00602") is not None


def test_Gogo_B_0101():
    # Gogo-B*0101 occurs 14 times in 2019 IEDB snapshot
    assert parse("Gogo-B*0101") is not None


def test_H2_class_I():
    # H2 class I occurs 5 times in 2019 IEDB snapshot
    assert parse("H2 class I") is not None


def test_H2_Db():
    # H2-Db occurs 6700 times in 2019 IEDB snapshot
    assert parse("H2-Db") is not None


def test_H2_Db_H155A_mutant():
    # H2-Db H155A mutant occurs 4 times in 2019 IEDB snapshot
    assert parse("H2-Db H155A mutant") is not None


def test_H2_Db_Q70H_mutant():
    # H2-Db Q70H mutant occurs 1 times in 2019 IEDB snapshot
    assert parse("H2-Db Q70H mutant") is not None


def test_H2_Db_Y159F_mutant():
    # H2-Db Y159F mutant occurs 3 times in 2019 IEDB snapshot
    assert parse("H2-Db Y159F mutant") is not None


def test_H2_Dd():
    # H2-Dd occurs 331 times in 2019 IEDB snapshot
    assert parse("H2-Dd") is not None


def test_H2_Dk():
    # H2-Dk occurs 8 times in 2019 IEDB snapshot
    assert parse("H2-Dk") is not None


def test_H2_IAb():
    # H2-IAb occurs 4683 times in 2019 IEDB snapshot
    assert parse("H2-IAb") is not None


def test_H2_IAb_I67F_R70Q_T71K_mutant():
    # H2-IAb I67F, R70Q, T71K mutant occurs 4 times in 2019 IEDB snapshot
    assert parse("H2-IAb I67F, R70Q, T71K mutant") is not None


def test_H2_IAd():
    # H2-IAd occurs 888 times in 2019 IEDB snapshot
    assert parse("H2-IAd") is not None


def test_H2_IAg7():
    # H2-IAg7 occurs 721 times in 2019 IEDB snapshot
    assert parse("H2-IAg7") is not None


def test_H2_IAg7_H56P_S57D_mutant():
    # H2-IAg7 H56P, S57D mutant occurs 52 times in 2019 IEDB snapshot
    assert parse("H2-IAg7 H56P, S57D mutant") is not None


def test_H2_IAk():
    # H2-IAk occurs 358 times in 2019 IEDB snapshot
    assert parse("H2-IAk") is not None


def test_H2_IAp():
    # H2-IAp occurs 20 times in 2019 IEDB snapshot
    assert parse("H2-IAp") is not None


def test_H2_IAq():
    # H2-IAq occurs 144 times in 2019 IEDB snapshot
    assert parse("H2-IAq") is not None


def test_H2_IAr():
    # H2-IAr occurs 24 times in 2019 IEDB snapshot
    assert parse("H2-IAr") is not None


def test_H2_IAs():
    # H2-IAs occurs 255 times in 2019 IEDB snapshot
    assert parse("H2-IAs") is not None


def test_H2_IAu():
    # H2-IAu occurs 231 times in 2019 IEDB snapshot
    assert parse("H2-IAu") is not None


def test_H2_IEb():
    # H2-IEb occurs 24 times in 2019 IEDB snapshot
    assert parse("H2-IEb") is not None


def test_H2_IEd():
    # H2-IEd occurs 431 times in 2019 IEDB snapshot
    assert parse("H2-IEd") is not None


def test_H2_IEg7():
    # H2-IEg7 occurs 262 times in 2019 IEDB snapshot
    assert parse("H2-IEg7") is not None


def test_H2_IEk():
    # H2-IEk occurs 319 times in 2019 IEDB snapshot
    assert parse("H2-IEk") is not None


def test_H2_IEk_alpha_E11Q_D66N_mutant():
    # H2-IEk alpha E11Q, D66N mutant occurs 4 times in 2019 IEDB snapshot
    assert parse("H2-IEk alpha E11Q, D66N mutant") is not None


def test_H2_IEp():
    # H2-IEp occurs 135 times in 2019 IEDB snapshot
    assert parse("H2-IEp") is not None


def test_H2_IEr():
    # H2-IEr occurs 7 times in 2019 IEDB snapshot
    assert parse("H2-IEr") is not None


def test_H2_Kb():
    # H2-Kb occurs 8710 times in 2019 IEDB snapshot
    assert parse("H2-Kb") is not None


def test_H2_Kb_D77S_K89A_mutant():
    # H2-Kb D77S, K89A mutant occurs 3 times in 2019 IEDB snapshot
    assert parse("H2-Kb D77S, K89A mutant") is not None


def test_H2_Kb_E152A_R155Y_L156Y_mutant():
    # H2-Kb E152A, R155Y, L156Y mutant occurs 8 times in 2019 IEDB snapshot
    assert parse("H2-Kb E152A, R155Y, L156Y mutant") is not None


def test_H2_Kb_Y22F_M23I_E24S_D30N_mutant():
    # H2-Kb Y22F, M23I, E24S, D30N mutant occurs 50 times in 2019 IEDB snapshot
    assert parse("H2-Kb Y22F, M23I, E24S, D30N mutant") is not None


def test_H2_Kb_Y84A_mutant():
    # H2-Kb Y84A mutant occurs 3 times in 2019 IEDB snapshot
    assert parse("H2-Kb Y84A mutant") is not None


def test_H2_Kb_Y84C_mutant():
    # H2-Kb Y84C mutant occurs 9 times in 2019 IEDB snapshot
    assert parse("H2-Kb Y84C mutant") is not None


def test_H2_Kd():
    # H2-Kd occurs 2076 times in 2019 IEDB snapshot
    assert parse("H2-Kd") is not None


def test_H2_Kk():
    # H2-Kk occurs 627 times in 2019 IEDB snapshot
    assert parse("H2-Kk") is not None


def test_H2_Kwm7():
    # H2-Kwm7 occurs 7 times in 2019 IEDB snapshot
    assert parse("H2-Kwm7") is not None


def test_H2_Ld():
    # H2-Ld occurs 487 times in 2019 IEDB snapshot
    assert parse("H2-Ld") is not None


def test_H2_Lq():
    # H2-Lq occurs 16 times in 2019 IEDB snapshot
    assert parse("H2-Lq") is not None


def test_H2_M3():
    # H2-M3 occurs 37 times in 2019 IEDB snapshot
    assert parse("H2-M3") is not None


def test_H2_Q1():
    # H2-Q1 occurs 235 times in 2019 IEDB snapshot
    assert parse("H2-Q1") is not None


def test_H2_Q2():
    # H2-Q2 occurs 682 times in 2019 IEDB snapshot
    assert parse("H2-Q2") is not None


def test_H2_Q9():
    # H2-Q9 occurs 11 times in 2019 IEDB snapshot
    assert parse("H2-Q9") is not None


def test_H2_Qa_1b():
    # H2-Qa-1b occurs 1 times in 2019 IEDB snapshot
    assert parse("H2-Qa-1b") is not None


def test_H2_b_class_I():
    # H2-b class I occurs 188 times in 2019 IEDB snapshot
    assert parse("H2-b class I") is not None


def test_H2_b_class_II():
    # H2-b class II occurs 11 times in 2019 IEDB snapshot
    assert parse("H2-b class II") is not None


def test_H2_d_class_I():
    # H2-d class I occurs 7 times in 2019 IEDB snapshot
    assert parse("H2-d class I") is not None


def test_H2_d_class_II():
    # H2-d class II occurs 4170 times in 2019 IEDB snapshot
    assert parse("H2-d class II") is not None


def test_H2_f_class_I():
    # H2-f class I occurs 2 times in 2019 IEDB snapshot
    assert parse("H2-f class I") is not None


def test_H2_f_class_II():
    # H2-f class II occurs 3 times in 2019 IEDB snapshot
    assert parse("H2-f class II") is not None


def test_H2_k_class_I():
    # H2-k class I occurs 2 times in 2019 IEDB snapshot
    assert parse("H2-k class I") is not None


def test_H2_k_class_II():
    # H2-k class II occurs 3 times in 2019 IEDB snapshot
    assert parse("H2-k class II") is not None


def test_H2_p_class_I():
    # H2-p class I occurs 2 times in 2019 IEDB snapshot
    assert parse("H2-p class I") is not None


def test_H2_q_class_I():
    # H2-q class I occurs 2 times in 2019 IEDB snapshot
    assert parse("H2-q class I") is not None


def test_H2_q_class_II():
    # H2-q class II occurs 3 times in 2019 IEDB snapshot
    assert parse("H2-q class II") is not None


def test_H2_r_class_I():
    # H2-r class I occurs 2 times in 2019 IEDB snapshot
    assert parse("H2-r class I") is not None


def test_H2_s_class_I():
    # H2-s class I occurs 2 times in 2019 IEDB snapshot
    assert parse("H2-s class I") is not None


def test_H2_s_class_II():
    # H2-s class II occurs 14 times in 2019 IEDB snapshot
    assert parse("H2-s class II") is not None


def test_H2_v_class_I():
    # H2-v class I occurs 2 times in 2019 IEDB snapshot
    assert parse("H2-v class I") is not None


def test_H2_z_class_I():
    # H2-z class I occurs 2 times in 2019 IEDB snapshot
    assert parse("H2-z class I") is not None


def test_H2_z_d_class_II():
    # H2-z/d class II occurs 253 times in 2019 IEDB snapshot
    assert parse("H2-z/d class II") is not None


def test_HLA_class_I():
    # HLA class I occurs 116121 times in 2019 IEDB snapshot
    assert parse("HLA class I") is not None


def test_HLA_class_II():
    # HLA class II occurs 509 times in 2019 IEDB snapshot
    assert parse("HLA class II") is not None


def test_HLA_A_01_01():
    # HLA-A*01:01 occurs 15962 times in 2019 IEDB snapshot
    assert parse("HLA-A*01:01") is not None


def test_HLA_A_01_03():
    # HLA-A*01:03 occurs 7 times in 2019 IEDB snapshot
    assert parse("HLA-A*01:03") is not None


def test_HLA_A_02_01():
    # HLA-A*02:01 occurs 45025 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:01") is not None


def test_HLA_A_02_01_A150P_mutant():
    # HLA-A*02:01 A150P mutant occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:01 A150P mutant") is not None


def test_HLA_A_02_01_K66A_mutant():
    # HLA-A*02:01 K66A mutant occurs 4 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:01 K66A mutant") is not None


def test_HLA_A_02_01_K66A_E63Q_mutant():
    # HLA-A*02:01 K66A, E63Q mutant occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:01 K66A, E63Q mutant") is not None


def test_HLA_A_02_01_T163A_mutant():
    # HLA-A*02:01 T163A mutant occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:01 T163A mutant") is not None


def test_HLA_A_02_01_W167A_mutant():
    # HLA-A*02:01 W167A mutant occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:01 W167A mutant") is not None


def test_HLA_A_02_02():
    # HLA-A*02:02 occurs 5298 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:02") is not None


def test_HLA_A_02_03():
    # HLA-A*02:03 occurs 8374 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:03") is not None


def test_HLA_A_02_04():
    # HLA-A*02:04 occurs 1684 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:04") is not None


def test_HLA_A_02_05():
    # HLA-A*02:05 occurs 420 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:05") is not None


def test_HLA_A_02_06():
    # HLA-A*02:06 occurs 5951 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:06") is not None


def test_HLA_A_02_07():
    # HLA-A*02:07 occurs 3567 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:07") is not None


def test_HLA_A_02_08():
    # HLA-A*02:08 occurs 5 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:08") is not None


def test_HLA_A_02_09():
    # HLA-A*02:09 occurs 5 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:09") is not None


def test_HLA_A_02_10():
    # HLA-A*02:10 occurs 18 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:10") is not None


def test_HLA_A_02_11():
    # HLA-A*02:11 occurs 718 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:11") is not None


def test_HLA_A_02_12():
    # HLA-A*02:12 occurs 892 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:12") is not None


def test_HLA_A_02_14():
    # HLA-A*02:14 occurs 12 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:14") is not None


def test_HLA_A_02_16():
    # HLA-A*02:16 occurs 604 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:16") is not None


def test_HLA_A_02_17():
    # HLA-A*02:17 occurs 332 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:17") is not None


def test_HLA_A_02_19():
    # HLA-A*02:19 occurs 836 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:19") is not None


def test_HLA_A_02_50():
    # HLA-A*02:50 occurs 125 times in 2019 IEDB snapshot
    assert parse("HLA-A*02:50") is not None


def test_HLA_A_03_01():
    # HLA-A*03:01 occurs 16821 times in 2019 IEDB snapshot
    assert parse("HLA-A*03:01") is not None


def test_HLA_A_03_02():
    # HLA-A*03:02 occurs 17 times in 2019 IEDB snapshot
    assert parse("HLA-A*03:02") is not None


def test_HLA_A_03_19():
    # HLA-A*03:19 occurs 29 times in 2019 IEDB snapshot
    assert parse("HLA-A*03:19") is not None


def test_HLA_A_11_01():
    # HLA-A*11:01 occurs 11248 times in 2019 IEDB snapshot
    assert parse("HLA-A*11:01") is not None


def test_HLA_A_11_02():
    # HLA-A*11:02 occurs 14 times in 2019 IEDB snapshot
    assert parse("HLA-A*11:02") is not None


def test_HLA_A_23_01():
    # HLA-A*23:01 occurs 1894 times in 2019 IEDB snapshot
    assert parse("HLA-A*23:01") is not None


def test_HLA_A_24_02():
    # HLA-A*24:02 occurs 12788 times in 2019 IEDB snapshot
    assert parse("HLA-A*24:02") is not None


def test_HLA_A_24_03():
    # HLA-A*24:03 occurs 1434 times in 2019 IEDB snapshot
    assert parse("HLA-A*24:03") is not None


def test_HLA_A_24_06():
    # HLA-A*24:06 occurs 407 times in 2019 IEDB snapshot
    assert parse("HLA-A*24:06") is not None


def test_HLA_A_24_13():
    # HLA-A*24:13 occurs 250 times in 2019 IEDB snapshot
    assert parse("HLA-A*24:13") is not None


def test_HLA_A_25_01():
    # HLA-A*25:01 occurs 912 times in 2019 IEDB snapshot
    assert parse("HLA-A*25:01") is not None


def test_HLA_A_26_01():
    # HLA-A*26:01 occurs 4190 times in 2019 IEDB snapshot
    assert parse("HLA-A*26:01") is not None


def test_HLA_A_26_02():
    # HLA-A*26:02 occurs 399 times in 2019 IEDB snapshot
    assert parse("HLA-A*26:02") is not None


def test_HLA_A_26_03():
    # HLA-A*26:03 occurs 269 times in 2019 IEDB snapshot
    assert parse("HLA-A*26:03") is not None


def test_HLA_A_29_02():
    # HLA-A*29:02 occurs 9741 times in 2019 IEDB snapshot
    assert parse("HLA-A*29:02") is not None


def test_HLA_A_30_01():
    # HLA-A*30:01 occurs 2234 times in 2019 IEDB snapshot
    assert parse("HLA-A*30:01") is not None


def test_HLA_A_30_02():
    # HLA-A*30:02 occurs 1944 times in 2019 IEDB snapshot
    assert parse("HLA-A*30:02") is not None


def test_HLA_A_30_03():
    # HLA-A*30:03 occurs 4 times in 2019 IEDB snapshot
    assert parse("HLA-A*30:03") is not None


def test_HLA_A_30_04():
    # HLA-A*30:04 occurs 5 times in 2019 IEDB snapshot
    assert parse("HLA-A*30:04") is not None


def test_HLA_A_30_14L():
    # HLA-A*30:14L occurs 33 times in 2019 IEDB snapshot
    assert parse("HLA-A*30:14L") is not None


def test_HLA_A_31_01():
    # HLA-A*31:01 occurs 6861 times in 2019 IEDB snapshot
    assert parse("HLA-A*31:01") is not None


def test_HLA_A_32_01():
    # HLA-A*32:01 occurs 1421 times in 2019 IEDB snapshot
    assert parse("HLA-A*32:01") is not None


def test_HLA_A_32_07():
    # HLA-A*32:07 occurs 87 times in 2019 IEDB snapshot
    assert parse("HLA-A*32:07") is not None


def test_HLA_A_32_15():
    # HLA-A*32:15 occurs 74 times in 2019 IEDB snapshot
    assert parse("HLA-A*32:15") is not None


def test_HLA_A_33_01():
    # HLA-A*33:01 occurs 2937 times in 2019 IEDB snapshot
    assert parse("HLA-A*33:01") is not None


def test_HLA_A_33_03():
    # HLA-A*33:03 occurs 22 times in 2019 IEDB snapshot
    assert parse("HLA-A*33:03") is not None


def test_HLA_A_66_01():
    # HLA-A*66:01 occurs 122 times in 2019 IEDB snapshot
    assert parse("HLA-A*66:01") is not None


def test_HLA_A_66_02():
    # HLA-A*66:02 occurs 31 times in 2019 IEDB snapshot
    assert parse("HLA-A*66:02") is not None


def test_HLA_A_68_01():
    # HLA-A*68:01 occurs 3552 times in 2019 IEDB snapshot
    assert parse("HLA-A*68:01") is not None


def test_HLA_A_68_02():
    # HLA-A*68:02 occurs 8820 times in 2019 IEDB snapshot
    assert parse("HLA-A*68:02") is not None


def test_HLA_A_68_23():
    # HLA-A*68:23 occurs 80 times in 2019 IEDB snapshot
    assert parse("HLA-A*68:23") is not None


def test_HLA_A_69_01():
    # HLA-A*69:01 occurs 3020 times in 2019 IEDB snapshot
    assert parse("HLA-A*69:01") is not None


def test_HLA_A_80_01():
    # HLA-A*80:01 occurs 1123 times in 2019 IEDB snapshot
    assert parse("HLA-A*80:01") is not None


def test_HLA_A1():
    # HLA-A1 occurs 291 times in 2019 IEDB snapshot
    assert parse("HLA-A1") is not None


def test_HLA_A11():
    # HLA-A11 occurs 759 times in 2019 IEDB snapshot
    assert parse("HLA-A11") is not None


def test_HLA_A2():
    # HLA-A2 occurs 2380 times in 2019 IEDB snapshot
    assert parse("HLA-A2") is not None


def test_HLA_A23():
    # HLA-A23 occurs 18 times in 2019 IEDB snapshot
    assert parse("HLA-A23") is not None


def test_HLA_A24():
    # HLA-A24 occurs 431 times in 2019 IEDB snapshot
    assert parse("HLA-A24") is not None


def test_HLA_A25():
    # HLA-A25 occurs 63 times in 2019 IEDB snapshot
    assert parse("HLA-A25") is not None


def test_HLA_A26():
    # HLA-A26 occurs 457 times in 2019 IEDB snapshot
    assert parse("HLA-A26") is not None


def test_HLA_A29():
    # HLA-A29 occurs 124 times in 2019 IEDB snapshot
    assert parse("HLA-A29") is not None


def test_HLA_A3():
    # HLA-A3 occurs 1685 times in 2019 IEDB snapshot
    assert parse("HLA-A3") is not None


def test_HLA_A30():
    # HLA-A30 occurs 44 times in 2019 IEDB snapshot
    assert parse("HLA-A30") is not None


def test_HLA_A31():
    # HLA-A31 occurs 23 times in 2019 IEDB snapshot
    assert parse("HLA-A31") is not None


def test_HLA_A32():
    # HLA-A32 occurs 115 times in 2019 IEDB snapshot
    assert parse("HLA-A32") is not None


def test_HLA_A33():
    # HLA-A33 occurs 21 times in 2019 IEDB snapshot
    assert parse("HLA-A33") is not None


def test_HLA_A68():
    # HLA-A68 occurs 53 times in 2019 IEDB snapshot
    assert parse("HLA-A68") is not None


def test_HLA_B_07_02():
    # HLA-B*07:02 occurs 20812 times in 2019 IEDB snapshot
    assert parse("HLA-B*07:02") is not None


def test_HLA_B_08_01():
    # HLA-B*08:01 occurs 7345 times in 2019 IEDB snapshot
    assert parse("HLA-B*08:01") is not None


def test_HLA_B_08_01_B_I66A_mutant():
    # HLA-B*08:01 B:I66A mutant occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-B*08:01 B:I66A mutant") is not None


def test_HLA_B_08_01_N80I_mutant():
    # HLA-B*08:01 N80I mutant occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-B*08:01 N80I mutant") is not None


def test_HLA_B_08_02():
    # HLA-B*08:02 occurs 431 times in 2019 IEDB snapshot
    assert parse("HLA-B*08:02") is not None


def test_HLA_B_08_03():
    # HLA-B*08:03 occurs 203 times in 2019 IEDB snapshot
    assert parse("HLA-B*08:03") is not None


def test_HLA_B_13_01():
    # HLA-B*13:01 occurs 54 times in 2019 IEDB snapshot
    assert parse("HLA-B*13:01") is not None


def test_HLA_B_13_02():
    # HLA-B*13:02 occurs 45 times in 2019 IEDB snapshot
    assert parse("HLA-B*13:02") is not None


def test_HLA_B_14_02():
    # HLA-B*14:02 occurs 440 times in 2019 IEDB snapshot
    assert parse("HLA-B*14:02") is not None


def test_HLA_B_14_03():
    # HLA-B*14:03 occurs 21 times in 2019 IEDB snapshot
    assert parse("HLA-B*14:03") is not None


def test_HLA_B_15_01():
    # HLA-B*15:01 occurs 16684 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:01") is not None


def test_HLA_B_15_02():
    # HLA-B*15:02 occurs 364 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:02") is not None


def test_HLA_B_15_03():
    # HLA-B*15:03 occurs 397 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:03") is not None


def test_HLA_B_15_08():
    # HLA-B*15:08 occurs 33 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:08") is not None


def test_HLA_B_15_09():
    # HLA-B*15:09 occurs 425 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:09") is not None


def test_HLA_B_15_10():
    # HLA-B*15:10 occurs 23 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:10") is not None


def test_HLA_B_15_11():
    # HLA-B*15:11 occurs 109 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:11") is not None


def test_HLA_B_15_13():
    # HLA-B*15:13 occurs 6 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:13") is not None


def test_HLA_B_15_16():
    # HLA-B*15:16 occurs 46 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:16") is not None


def test_HLA_B_15_17():
    # HLA-B*15:17 occurs 1412 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:17") is not None


def test_HLA_B_15_18():
    # HLA-B*15:18 occurs 5 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:18") is not None


def test_HLA_B_15_42():
    # HLA-B*15:42 occurs 362 times in 2019 IEDB snapshot
    assert parse("HLA-B*15:42") is not None


def test_HLA_B_18_01():
    # HLA-B*18:01 occurs 3468 times in 2019 IEDB snapshot
    assert parse("HLA-B*18:01") is not None


def test_HLA_B_18_03():
    # HLA-B*18:03 occurs 215 times in 2019 IEDB snapshot
    assert parse("HLA-B*18:03") is not None


def test_HLA_B_27_01():
    # HLA-B*27:01 occurs 3944 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:01") is not None


def test_HLA_B_27_02():
    # HLA-B*27:02 occurs 3682 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:02") is not None


def test_HLA_B_27_03():
    # HLA-B*27:03 occurs 2150 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:03") is not None


def test_HLA_B_27_04():
    # HLA-B*27:04 occurs 1637 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:04") is not None


def test_HLA_B_27_05():
    # HLA-B*27:05 occurs 95076 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:05") is not None


def test_HLA_B_27_05_D74Y_D77N_mutant():
    # HLA-B*27:05 D74Y D77N mutant occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:05 D74Y D77N mutant") is not None


def test_HLA_B_27_05_L81A_mutant():
    # HLA-B*27:05 L81A mutant occurs 4 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:05 L81A mutant") is not None


def test_HLA_B_27_06():
    # HLA-B*27:06 occurs 1540 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:06") is not None


def test_HLA_B_27_07():
    # HLA-B*27:07 occurs 2232 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:07") is not None


def test_HLA_B_27_08():
    # HLA-B*27:08 occurs 2354 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:08") is not None


def test_HLA_B_27_09():
    # HLA-B*27:09 occurs 2515 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:09") is not None


def test_HLA_B_27_10():
    # HLA-B*27:10 occurs 13 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:10") is not None


def test_HLA_B_27_20():
    # HLA-B*27:20 occurs 87 times in 2019 IEDB snapshot
    assert parse("HLA-B*27:20") is not None


def test_HLA_B_35_01():
    # HLA-B*35:01 occurs 5427 times in 2019 IEDB snapshot
    assert parse("HLA-B*35:01") is not None


def test_HLA_B_35_02():
    # HLA-B*35:02 occurs 38 times in 2019 IEDB snapshot
    assert parse("HLA-B*35:02") is not None


def test_HLA_B_35_03():
    # HLA-B*35:03 occurs 256 times in 2019 IEDB snapshot
    assert parse("HLA-B*35:03") is not None


def test_HLA_B_35_04():
    # HLA-B*35:04 occurs 12 times in 2019 IEDB snapshot
    assert parse("HLA-B*35:04") is not None


def test_HLA_B_35_06():
    # HLA-B*35:06 occurs 10 times in 2019 IEDB snapshot
    assert parse("HLA-B*35:06") is not None


def test_HLA_B_35_08():
    # HLA-B*35:08 occurs 432 times in 2019 IEDB snapshot
    assert parse("HLA-B*35:08") is not None


def test_HLA_B_35_08_Q65A_T69A_Q155A_mutant():
    # HLA-B*35:08 Q65A, T69A, Q155A mutant occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-B*35:08 Q65A, T69A, Q155A mutant") is not None


def test_HLA_B_37_01():
    # HLA-B*37:01 occurs 39 times in 2019 IEDB snapshot
    assert parse("HLA-B*37:01") is not None


def test_HLA_B_38_01():
    # HLA-B*38:01 occurs 158 times in 2019 IEDB snapshot
    assert parse("HLA-B*38:01") is not None


def test_HLA_B_39_01():
    # HLA-B*39:01 occurs 2250 times in 2019 IEDB snapshot
    assert parse("HLA-B*39:01") is not None


def test_HLA_B_39_05():
    # HLA-B*39:05 occurs 8 times in 2019 IEDB snapshot
    assert parse("HLA-B*39:05") is not None


def test_HLA_B_39_06():
    # HLA-B*39:06 occurs 54 times in 2019 IEDB snapshot
    assert parse("HLA-B*39:06") is not None


def test_HLA_B_39_09():
    # HLA-B*39:09 occurs 9 times in 2019 IEDB snapshot
    assert parse("HLA-B*39:09") is not None


def test_HLA_B_39_10():
    # HLA-B*39:10 occurs 6 times in 2019 IEDB snapshot
    assert parse("HLA-B*39:10") is not None


def test_HLA_B_39_24():
    # HLA-B*39:24 occurs 261 times in 2019 IEDB snapshot
    assert parse("HLA-B*39:24") is not None


def test_HLA_B_40_01():
    # HLA-B*40:01 occurs 6071 times in 2019 IEDB snapshot
    assert parse("HLA-B*40:01") is not None


def test_HLA_B_40_02():
    # HLA-B*40:02 occurs 9574 times in 2019 IEDB snapshot
    assert parse("HLA-B*40:02") is not None


def test_HLA_B_40_13():
    # HLA-B*40:13 occurs 56 times in 2019 IEDB snapshot
    assert parse("HLA-B*40:13") is not None


def test_HLA_B_41_01():
    # HLA-B*41:01 occurs 486 times in 2019 IEDB snapshot
    assert parse("HLA-B*41:01") is not None


def test_HLA_B_41_02():
    # HLA-B*41:02 occurs 22 times in 2019 IEDB snapshot
    assert parse("HLA-B*41:02") is not None


def test_HLA_B_41_03():
    # HLA-B*41:03 occurs 86 times in 2019 IEDB snapshot
    assert parse("HLA-B*41:03") is not None


def test_HLA_B_41_04():
    # HLA-B*41:04 occurs 53 times in 2019 IEDB snapshot
    assert parse("HLA-B*41:04") is not None


def test_HLA_B_41_05():
    # HLA-B*41:05 occurs 41 times in 2019 IEDB snapshot
    assert parse("HLA-B*41:05") is not None


def test_HLA_B_41_06():
    # HLA-B*41:06 occurs 22 times in 2019 IEDB snapshot
    assert parse("HLA-B*41:06") is not None


def test_HLA_B_42_01():
    # HLA-B*42:01 occurs 21 times in 2019 IEDB snapshot
    assert parse("HLA-B*42:01") is not None


def test_HLA_B_42_02():
    # HLA-B*42:02 occurs 5 times in 2019 IEDB snapshot
    assert parse("HLA-B*42:02") is not None


def test_HLA_B_44_02():
    # HLA-B*44:02 occurs 9186 times in 2019 IEDB snapshot
    assert parse("HLA-B*44:02") is not None


def test_HLA_B_44_03():
    # HLA-B*44:03 occurs 11405 times in 2019 IEDB snapshot
    assert parse("HLA-B*44:03") is not None


def test_HLA_B_44_05():
    # HLA-B*44:05 occurs 25 times in 2019 IEDB snapshot
    assert parse("HLA-B*44:05") is not None


def test_HLA_B_44_08():
    # HLA-B*44:08 occurs 50 times in 2019 IEDB snapshot
    assert parse("HLA-B*44:08") is not None


def test_HLA_B_44_27():
    # HLA-B*44:27 occurs 207 times in 2019 IEDB snapshot
    assert parse("HLA-B*44:27") is not None


def test_HLA_B_44_28():
    # HLA-B*44:28 occurs 131 times in 2019 IEDB snapshot
    assert parse("HLA-B*44:28") is not None


def test_HLA_B_45_01():
    # HLA-B*45:01 occurs 1365 times in 2019 IEDB snapshot
    assert parse("HLA-B*45:01") is not None


def test_HLA_B_45_06():
    # HLA-B*45:06 occurs 355 times in 2019 IEDB snapshot
    assert parse("HLA-B*45:06") is not None


def test_HLA_B_46_01():
    # HLA-B*46:01 occurs 3822 times in 2019 IEDB snapshot
    assert parse("HLA-B*46:01") is not None


def test_HLA_B_47_01():
    # HLA-B*47:01 occurs 27 times in 2019 IEDB snapshot
    assert parse("HLA-B*47:01") is not None


def test_HLA_B_48_01():
    # HLA-B*48:01 occurs 877 times in 2019 IEDB snapshot
    assert parse("HLA-B*48:01") is not None


def test_HLA_B_49_01():
    # HLA-B*49:01 occurs 208 times in 2019 IEDB snapshot
    assert parse("HLA-B*49:01") is not None


def test_HLA_B_50_01():
    # HLA-B*50:01 occurs 694 times in 2019 IEDB snapshot
    assert parse("HLA-B*50:01") is not None


def test_HLA_B_50_02():
    # HLA-B*50:02 occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-B*50:02") is not None


def test_HLA_B_51_01():
    # HLA-B*51:01 occurs 6591 times in 2019 IEDB snapshot
    assert parse("HLA-B*51:01") is not None


def test_HLA_B_51_02():
    # HLA-B*51:02 occurs 7 times in 2019 IEDB snapshot
    assert parse("HLA-B*51:02") is not None


def test_HLA_B_51_08():
    # HLA-B*51:08 occurs 624 times in 2019 IEDB snapshot
    assert parse("HLA-B*51:08") is not None


def test_HLA_B_52_01():
    # HLA-B*52:01 occurs 10 times in 2019 IEDB snapshot
    assert parse("HLA-B*52:01") is not None


def test_HLA_B_53_01():
    # HLA-B*53:01 occurs 1248 times in 2019 IEDB snapshot
    assert parse("HLA-B*53:01") is not None


def test_HLA_B_54_01():
    # HLA-B*54:01 occurs 1903 times in 2019 IEDB snapshot
    assert parse("HLA-B*54:01") is not None


def test_HLA_B_55_01():
    # HLA-B*55:01 occurs 4 times in 2019 IEDB snapshot
    assert parse("HLA-B*55:01") is not None


def test_HLA_B_55_02():
    # HLA-B*55:02 occurs 28 times in 2019 IEDB snapshot
    assert parse("HLA-B*55:02") is not None


def test_HLA_B_56_01():
    # HLA-B*56:01 occurs 728 times in 2019 IEDB snapshot
    assert parse("HLA-B*56:01") is not None


def test_HLA_B_57_01():
    # HLA-B*57:01 occurs 17139 times in 2019 IEDB snapshot
    assert parse("HLA-B*57:01") is not None


def test_HLA_B_57_01_I80N_mutant():
    # HLA-B*57:01 I80N mutant occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-B*57:01 I80N mutant") is not None


def test_HLA_B_57_01_I80T_mutant():
    # HLA-B*57:01 I80T mutant occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-B*57:01 I80T mutant") is not None


def test_HLA_B_57_02():
    # HLA-B*57:02 occurs 8 times in 2019 IEDB snapshot
    assert parse("HLA-B*57:02") is not None


def test_HLA_B_57_03():
    # HLA-B*57:03 occurs 32 times in 2019 IEDB snapshot
    assert parse("HLA-B*57:03") is not None


def test_HLA_B_58_01():
    # HLA-B*58:01 occurs 4026 times in 2019 IEDB snapshot
    assert parse("HLA-B*58:01") is not None


def test_HLA_B_58_02():
    # HLA-B*58:02 occurs 7 times in 2019 IEDB snapshot
    assert parse("HLA-B*58:02") is not None


def test_HLA_B_73_01():
    # HLA-B*73:01 occurs 306 times in 2019 IEDB snapshot
    assert parse("HLA-B*73:01") is not None


def test_HLA_B_81_01():
    # HLA-B*81:01 occurs 7 times in 2019 IEDB snapshot
    assert parse("HLA-B*81:01") is not None


def test_HLA_B_83_01():
    # HLA-B*83:01 occurs 333 times in 2019 IEDB snapshot
    assert parse("HLA-B*83:01") is not None


def test_HLA_B12():
    # HLA-B12 occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-B12") is not None


def test_HLA_B13():
    # HLA-B13 occurs 43 times in 2019 IEDB snapshot
    assert parse("HLA-B13") is not None


def test_HLA_B15():
    # HLA-B15 occurs 20 times in 2019 IEDB snapshot
    assert parse("HLA-B15") is not None


def test_HLA_B18():
    # HLA-B18 occurs 95 times in 2019 IEDB snapshot
    assert parse("HLA-B18") is not None


def test_HLA_B27():
    # HLA-B27 occurs 1883 times in 2019 IEDB snapshot
    assert parse("HLA-B27") is not None


def test_HLA_B35():
    # HLA-B35 occurs 913 times in 2019 IEDB snapshot
    assert parse("HLA-B35") is not None


def test_HLA_B37():
    # HLA-B37 occurs 131 times in 2019 IEDB snapshot
    assert parse("HLA-B37") is not None


def test_HLA_B38():
    # HLA-B38 occurs 48 times in 2019 IEDB snapshot
    assert parse("HLA-B38") is not None


def test_HLA_B39():
    # HLA-B39 occurs 1343 times in 2019 IEDB snapshot
    assert parse("HLA-B39") is not None


def test_HLA_B40():
    # HLA-B40 occurs 1632 times in 2019 IEDB snapshot
    assert parse("HLA-B40") is not None


def test_HLA_B44():
    # HLA-B44 occurs 727 times in 2019 IEDB snapshot
    assert parse("HLA-B44") is not None


def test_HLA_B45():
    # HLA-B45 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-B45") is not None


def test_HLA_B49():
    # HLA-B49 occurs 7 times in 2019 IEDB snapshot
    assert parse("HLA-B49") is not None


def test_HLA_B50():
    # HLA-B50 occurs 10 times in 2019 IEDB snapshot
    assert parse("HLA-B50") is not None


def test_HLA_B51():
    # HLA-B51 occurs 545 times in 2019 IEDB snapshot
    assert parse("HLA-B51") is not None


def test_HLA_B53():
    # HLA-B53 occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-B53") is not None


def test_HLA_B54():
    # HLA-B54 occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-B54") is not None


def test_HLA_B55():
    # HLA-B55 occurs 14 times in 2019 IEDB snapshot
    assert parse("HLA-B55") is not None


def test_HLA_B57():
    # HLA-B57 occurs 398 times in 2019 IEDB snapshot
    assert parse("HLA-B57") is not None


def test_HLA_B58():
    # HLA-B58 occurs 8 times in 2019 IEDB snapshot
    assert parse("HLA-B58") is not None


def test_HLA_B60():
    # HLA-B60 occurs 6 times in 2019 IEDB snapshot
    assert parse("HLA-B60") is not None


def test_HLA_B61():
    # HLA-B61 occurs 15 times in 2019 IEDB snapshot
    assert parse("HLA-B61") is not None


def test_HLA_B62():
    # HLA-B62 occurs 37 times in 2019 IEDB snapshot
    assert parse("HLA-B62") is not None


def test_HLA_B7():
    # HLA-B7 occurs 6241 times in 2019 IEDB snapshot
    assert parse("HLA-B7") is not None


def test_HLA_B78():
    # HLA-B78 occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-B78") is not None


def test_HLA_B8():
    # HLA-B8 occurs 263 times in 2019 IEDB snapshot
    assert parse("HLA-B8") is not None


def test_HLA_C_01_02():
    # HLA-C*01:02 occurs 732 times in 2019 IEDB snapshot
    assert parse("HLA-C*01:02") is not None


def test_HLA_C_03_03():
    # HLA-C*03:03 occurs 235 times in 2019 IEDB snapshot
    assert parse("HLA-C*03:03") is not None


def test_HLA_C_03_04():
    # HLA-C*03:04 occurs 500 times in 2019 IEDB snapshot
    assert parse("HLA-C*03:04") is not None


def test_HLA_C_04_01():
    # HLA-C*04:01 occurs 1700 times in 2019 IEDB snapshot
    assert parse("HLA-C*04:01") is not None


def test_HLA_C_05_01():
    # HLA-C*05:01 occurs 2920 times in 2019 IEDB snapshot
    assert parse("HLA-C*05:01") is not None


def test_HLA_C_06_02():
    # HLA-C*06:02 occurs 3080 times in 2019 IEDB snapshot
    assert parse("HLA-C*06:02") is not None


def test_HLA_C_07_01():
    # HLA-C*07:01 occurs 495 times in 2019 IEDB snapshot
    assert parse("HLA-C*07:01") is not None


def test_HLA_C_07_02():
    # HLA-C*07:02 occurs 734 times in 2019 IEDB snapshot
    assert parse("HLA-C*07:02") is not None


def test_HLA_C_07_04():
    # HLA-C*07:04 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-C*07:04") is not None


def test_HLA_C_08_01():
    # HLA-C*08:01 occurs 48 times in 2019 IEDB snapshot
    assert parse("HLA-C*08:01") is not None


def test_HLA_C_08_02():
    # HLA-C*08:02 occurs 135 times in 2019 IEDB snapshot
    assert parse("HLA-C*08:02") is not None


def test_HLA_C_12_02():
    # HLA-C*12:02 occurs 10 times in 2019 IEDB snapshot
    assert parse("HLA-C*12:02") is not None


def test_HLA_C_12_03():
    # HLA-C*12:03 occurs 185 times in 2019 IEDB snapshot
    assert parse("HLA-C*12:03") is not None


def test_HLA_C_12_04():
    # HLA-C*12:04 occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-C*12:04") is not None


def test_HLA_C_14_02():
    # HLA-C*14:02 occurs 320 times in 2019 IEDB snapshot
    assert parse("HLA-C*14:02") is not None


def test_HLA_C_15_02():
    # HLA-C*15:02 occurs 181 times in 2019 IEDB snapshot
    assert parse("HLA-C*15:02") is not None


def test_HLA_C_16_01():
    # HLA-C*16:01 occurs 774 times in 2019 IEDB snapshot
    assert parse("HLA-C*16:01") is not None


def test_HLA_Cw1():
    # HLA-Cw1 occurs 17 times in 2019 IEDB snapshot
    assert parse("HLA-Cw1") is not None


def test_HLA_Cw12():
    # HLA-Cw12 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-Cw12") is not None


def test_HLA_Cw3():
    # HLA-Cw3 occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-Cw3") is not None


def test_HLA_Cw4():
    # HLA-Cw4 occurs 38 times in 2019 IEDB snapshot
    assert parse("HLA-Cw4") is not None


def test_HLA_Cw7():
    # HLA-Cw7 occurs 14 times in 2019 IEDB snapshot
    assert parse("HLA-Cw7") is not None


def test_HLA_DP():
    # HLA-DP occurs 10 times in 2019 IEDB snapshot
    assert parse("HLA-DP") is not None


def test_HLA_DPA1_01_03_DPB1_02_01():
    # HLA-DPA1*01:03/DPB1*02:01 occurs 850 times in 2019 IEDB snapshot
    assert parse("HLA-DPA1*01:03/DPB1*02:01") is not None


def test_HLA_DPA1_01_03_DPB1_03_01():
    # HLA-DPA1*01:03/DPB1*03:01 occurs 176 times in 2019 IEDB snapshot
    assert parse("HLA-DPA1*01:03/DPB1*03:01") is not None


def test_HLA_DPA1_01_03_DPB1_04_01():
    # HLA-DPA1*01:03/DPB1*04:01 occurs 984 times in 2019 IEDB snapshot
    assert parse("HLA-DPA1*01:03/DPB1*04:01") is not None


def test_HLA_DPA1_01_03_DPB1_04_02():
    # HLA-DPA1*01:03/DPB1*04:02 occurs 44 times in 2019 IEDB snapshot
    assert parse("HLA-DPA1*01:03/DPB1*04:02") is not None


def test_HLA_DPA1_02_01_DPB1_01_01():
    # HLA-DPA1*02:01/DPB1*01:01 occurs 983 times in 2019 IEDB snapshot
    assert parse("HLA-DPA1*02:01/DPB1*01:01") is not None


def test_HLA_DPA1_02_01_DPB1_05_01():
    # HLA-DPA1*02:01/DPB1*05:01 occurs 972 times in 2019 IEDB snapshot
    assert parse("HLA-DPA1*02:01/DPB1*05:01") is not None


def test_HLA_DPA1_02_02_DPB1_05_01():
    # HLA-DPA1*02:02/DPB1*05:01 occurs 7 times in 2019 IEDB snapshot
    assert parse("HLA-DPA1*02:02/DPB1*05:01") is not None


def test_HLA_DPA1_03_01_DPB1_04_02():
    # HLA-DPA1*03:01/DPB1*04:02 occurs 974 times in 2019 IEDB snapshot
    assert parse("HLA-DPA1*03:01/DPB1*04:02") is not None


def test_HLA_DPB1_01_01():
    # HLA-DPB1*01:01 occurs 106 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*01:01") is not None


def test_HLA_DPB1_02_01():
    # HLA-DPB1*02:01 occurs 301 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*02:01") is not None


def test_HLA_DPB1_02_01_E69K_mutant():
    # HLA-DPB1*02:01 E69K mutant occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*02:01 E69K mutant") is not None


def test_HLA_DPB1_03_01():
    # HLA-DPB1*03:01 occurs 117 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*03:01") is not None


def test_HLA_DPB1_04_01():
    # HLA-DPB1*04:01 occurs 357 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*04:01") is not None


def test_HLA_DPB1_04_02():
    # HLA-DPB1*04:02 occurs 317 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*04:02") is not None


def test_HLA_DPB1_05_01():
    # HLA-DPB1*05:01 occurs 124 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*05:01") is not None


def test_HLA_DPB1_09_01():
    # HLA-DPB1*09:01 occurs 65 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*09:01") is not None


def test_HLA_DPB1_11_01():
    # HLA-DPB1*11:01 occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*11:01") is not None


def test_HLA_DPB1_14_01():
    # HLA-DPB1*14:01 occurs 285 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*14:01") is not None


def test_HLA_DPB1_20_01():
    # HLA-DPB1*20:01 occurs 278 times in 2019 IEDB snapshot
    assert parse("HLA-DPB1*20:01") is not None


def test_HLA_DPw2():
    # HLA-DPw2 occurs 78 times in 2019 IEDB snapshot
    assert parse("HLA-DPw2") is not None


def test_HLA_DPw4():
    # HLA-DPw4 occurs 12 times in 2019 IEDB snapshot
    assert parse("HLA-DPw4") is not None


def test_HLA_DQ():
    # HLA-DQ occurs 165 times in 2019 IEDB snapshot
    assert parse("HLA-DQ") is not None


def test_HLA_DQ1():
    # HLA-DQ1 occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-DQ1") is not None


def test_HLA_DQ2():
    # HLA-DQ2 occurs 281 times in 2019 IEDB snapshot
    assert parse("HLA-DQ2") is not None


def test_HLA_DQ5():
    # HLA-DQ5 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-DQ5") is not None


def test_HLA_DQ6():
    # HLA-DQ6 occurs 26 times in 2019 IEDB snapshot
    assert parse("HLA-DQ6") is not None


def test_HLA_DQ7():
    # HLA-DQ7 occurs 18 times in 2019 IEDB snapshot
    assert parse("HLA-DQ7") is not None


def test_HLA_DQ8():
    # HLA-DQ8 occurs 214 times in 2019 IEDB snapshot
    assert parse("HLA-DQ8") is not None


def test_HLA_DQ9():
    # HLA-DQ9 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-DQ9") is not None


def test_HLA_DQA1_01_01():
    # HLA-DQA1*01:01 occurs 90 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:01") is not None


def test_HLA_DQA1_01_01_DQB1_05_01():
    # HLA-DQA1*01:01/DQB1*05:01 occurs 1023 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:01/DQB1*05:01") is not None


def test_HLA_DQA1_01_01_DQB1_05_03():
    # HLA-DQA1*01:01/DQB1*05:03 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:01/DQB1*05:03") is not None


def test_HLA_DQA1_01_02():
    # HLA-DQA1*01:02 occurs 90 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:02") is not None


def test_HLA_DQA1_01_02_DQB1_05_01():
    # HLA-DQA1*01:02/DQB1*05:01 occurs 839 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:02/DQB1*05:01") is not None


def test_HLA_DQA1_01_02_DQB1_05_02():
    # HLA-DQA1*01:02/DQB1*05:02 occurs 176 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:02/DQB1*05:02") is not None


def test_HLA_DQA1_01_02_DQB1_06_02():
    # HLA-DQA1*01:02/DQB1*06:02 occurs 1189 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:02/DQB1*06:02") is not None


def test_HLA_DQA1_01_02_DQB1_06_04():
    # HLA-DQA1*01:02/DQB1*06:04 occurs 111 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:02/DQB1*06:04") is not None


def test_HLA_DQA1_01_03_DQB1_03_02():
    # HLA-DQA1*01:03/DQB1*03:02 occurs 6 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:03/DQB1*03:02") is not None


def test_HLA_DQA1_01_03_DQB1_06_01():
    # HLA-DQA1*01:03/DQB1*06:01 occurs 4 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:03/DQB1*06:01") is not None


def test_HLA_DQA1_01_03_DQB1_06_03():
    # HLA-DQA1*01:03/DQB1*06:03 occurs 378 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:03/DQB1*06:03") is not None


def test_HLA_DQA1_01_04_DQB1_05_03():
    # HLA-DQA1*01:04/DQB1*05:03 occurs 169 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*01:04/DQB1*05:03") is not None


def test_HLA_DQA1_02_01_DQB1_02_01():
    # HLA-DQA1*02:01/DQB1*02:01 occurs 94 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*02:01/DQB1*02:01") is not None


def test_HLA_DQA1_02_01_DQB1_02_02():
    # HLA-DQA1*02:01/DQB1*02:02 occurs 7584 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*02:01/DQB1*02:02") is not None


def test_HLA_DQA1_02_01_DQB1_03_01():
    # HLA-DQA1*02:01/DQB1*03:01 occurs 829 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*02:01/DQB1*03:01") is not None


def test_HLA_DQA1_02_01_DQB1_03_03():
    # HLA-DQA1*02:01/DQB1*03:03 occurs 764 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*02:01/DQB1*03:03") is not None


def test_HLA_DQA1_02_01_DQB1_04_02():
    # HLA-DQA1*02:01/DQB1*04:02 occurs 768 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*02:01/DQB1*04:02") is not None


def test_HLA_DQA1_03_01():
    # HLA-DQA1*03:01 occurs 90 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:01") is not None


def test_HLA_DQA1_03_01_DQB1_02_01():
    # HLA-DQA1*03:01/DQB1*02:01 occurs 28 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:01/DQB1*02:01") is not None


def test_HLA_DQA1_03_01_DQB1_03_01():
    # HLA-DQA1*03:01/DQB1*03:01 occurs 139 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:01/DQB1*03:01") is not None


def test_HLA_DQA1_03_01_DQB1_03_02():
    # HLA-DQA1*03:01/DQB1*03:02 occurs 1298 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:01/DQB1*03:02") is not None


def test_HLA_DQA1_03_01_DQB1_04_01():
    # HLA-DQA1*03:01/DQB1*04:01 occurs 5 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:01/DQB1*04:01") is not None


def test_HLA_DQA1_03_02_DQB1_03_01():
    # HLA-DQA1*03:02/DQB1*03:01 occurs 24 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:02/DQB1*03:01") is not None


def test_HLA_DQA1_03_02_DQB1_03_03():
    # HLA-DQA1*03:02/DQB1*03:03 occurs 10 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:02/DQB1*03:03") is not None


def test_HLA_DQA1_03_02_DQB1_04_01():
    # HLA-DQA1*03:02/DQB1*04:01 occurs 154 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:02/DQB1*04:01") is not None


def test_HLA_DQA1_03_03_DQB1_04_02():
    # HLA-DQA1*03:03/DQB1*04:02 occurs 567 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*03:03/DQB1*04:02") is not None


def test_HLA_DQA1_04_01_DQB1_04_02():
    # HLA-DQA1*04:01/DQB1*04:02 occurs 976 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*04:01/DQB1*04:02") is not None


def test_HLA_DQA1_05_01():
    # HLA-DQA1*05:01 occurs 90 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*05:01") is not None


def test_HLA_DQA1_05_01_DQB1_02_01():
    # HLA-DQA1*05:01/DQB1*02:01 occurs 5922 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*05:01/DQB1*02:01") is not None


def test_HLA_DQA1_05_01_DQB1_03_01():
    # HLA-DQA1*05:01/DQB1*03:01 occurs 1009 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*05:01/DQB1*03:01") is not None


def test_HLA_DQA1_05_01_DQB1_03_02():
    # HLA-DQA1*05:01/DQB1*03:02 occurs 881 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*05:01/DQB1*03:02") is not None


def test_HLA_DQA1_05_01_DQB1_03_03():
    # HLA-DQA1*05:01/DQB1*03:03 occurs 564 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*05:01/DQB1*03:03") is not None


def test_HLA_DQA1_05_01_DQB1_04_02():
    # HLA-DQA1*05:01/DQB1*04:02 occurs 749 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*05:01/DQB1*04:02") is not None


def test_HLA_DQA1_05_05_DQB1_03_01():
    # HLA-DQA1*05:05/DQB1*03:01 occurs 7441 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*05:05/DQB1*03:01") is not None


def test_HLA_DQA1_06_01_DQB1_04_02():
    # HLA-DQA1*06:01/DQB1*04:02 occurs 565 times in 2019 IEDB snapshot
    assert parse("HLA-DQA1*06:01/DQB1*04:02") is not None


def test_HLA_DQB1_02_01():
    # HLA-DQB1*02:01 occurs 284 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*02:01") is not None


def test_HLA_DQB1_02_02():
    # HLA-DQB1*02:02 occurs 102 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*02:02") is not None


def test_HLA_DQB1_03_01():
    # HLA-DQB1*03:01 occurs 313 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*03:01") is not None


def test_HLA_DQB1_03_02():
    # HLA-DQB1*03:02 occurs 549 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*03:02") is not None


def test_HLA_DQB1_03_19():
    # HLA-DQB1*03:19 occurs 6 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*03:19") is not None


def test_HLA_DQB1_04_02():
    # HLA-DQB1*04:02 occurs 113 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*04:02") is not None


def test_HLA_DQB1_05_01():
    # HLA-DQB1*05:01 occurs 210 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*05:01") is not None


def test_HLA_DQB1_05_02():
    # HLA-DQB1*05:02 occurs 102 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*05:02") is not None


def test_HLA_DQB1_05_03():
    # HLA-DQB1*05:03 occurs 102 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*05:03") is not None


def test_HLA_DQB1_06_02():
    # HLA-DQB1*06:02 occurs 399 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*06:02") is not None


def test_HLA_DQB1_06_03():
    # HLA-DQB1*06:03 occurs 2 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*06:03") is not None


def test_HLA_DQB1_06_04():
    # HLA-DQB1*06:04 occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-DQB1*06:04") is not None


def test_HLA_DR():
    # HLA-DR occurs 14136 times in 2019 IEDB snapshot
    assert parse("HLA-DR") is not None


def test_HLA_DR1():
    # HLA-DR1 occurs 6290 times in 2019 IEDB snapshot
    assert parse("HLA-DR1") is not None


def test_HLA_DR11():
    # HLA-DR11 occurs 417 times in 2019 IEDB snapshot
    assert parse("HLA-DR11") is not None


def test_HLA_DR12():
    # HLA-DR12 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-DR12") is not None


def test_HLA_DR13():
    # HLA-DR13 occurs 354 times in 2019 IEDB snapshot
    assert parse("HLA-DR13") is not None


def test_HLA_DR14():
    # HLA-DR14 occurs 100 times in 2019 IEDB snapshot
    assert parse("HLA-DR14") is not None


def test_HLA_DR15():
    # HLA-DR15 occurs 302 times in 2019 IEDB snapshot
    assert parse("HLA-DR15") is not None


def test_HLA_DR17():
    # HLA-DR17 occurs 80 times in 2019 IEDB snapshot
    assert parse("HLA-DR17") is not None


def test_HLA_DR2():
    # HLA-DR2 occurs 452 times in 2019 IEDB snapshot
    assert parse("HLA-DR2") is not None


def test_HLA_DR3():
    # HLA-DR3 occurs 900 times in 2019 IEDB snapshot
    assert parse("HLA-DR3") is not None


def test_HLA_DR4():
    # HLA-DR4 occurs 735 times in 2019 IEDB snapshot
    assert parse("HLA-DR4") is not None


def test_HLA_DR5():
    # HLA-DR5 occurs 56 times in 2019 IEDB snapshot
    assert parse("HLA-DR5") is not None


def test_HLA_DR52():
    # HLA-DR52 occurs 15 times in 2019 IEDB snapshot
    assert parse("HLA-DR52") is not None


def test_HLA_DR53():
    # HLA-DR53 occurs 21 times in 2019 IEDB snapshot
    assert parse("HLA-DR53") is not None


def test_HLA_DR6():
    # HLA-DR6 occurs 27 times in 2019 IEDB snapshot
    assert parse("HLA-DR6") is not None


def test_HLA_DR7():
    # HLA-DR7 occurs 586 times in 2019 IEDB snapshot
    assert parse("HLA-DR7") is not None


def test_HLA_DR8():
    # HLA-DR8 occurs 115 times in 2019 IEDB snapshot
    assert parse("HLA-DR8") is not None


def test_HLA_DR9():
    # HLA-DR9 occurs 34 times in 2019 IEDB snapshot
    assert parse("HLA-DR9") is not None


def test_HLA_DRA_01_01_F54C_mutant_DRB1_01_01():
    # HLA-DRA*01:01 F54C mutant/DRB1*01:01 occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01 F54C mutant/DRB1*01:01") is not None


def test_HLA_DRA_01_01_DRB1_01_01():
    # HLA-DRA*01:01/DRB1*01:01 occurs 251 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*01:01") is not None


def test_HLA_DRA_01_01_DRB1_01_01_C30S_mutant():
    # HLA-DRA*01:01/DRB1*01:01 C30S mutant occurs 888 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*01:01 C30S mutant") is not None


def test_HLA_DRA_01_01_DRB1_01_01_G86Y_mutant():
    # HLA-DRA*01:01/DRB1*01:01 G86Y mutant occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*01:01 G86Y mutant") is not None


def test_HLA_DRA_01_01_DRB1_03_01():
    # HLA-DRA*01:01/DRB1*03:01 occurs 974 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*03:01") is not None


def test_HLA_DRA_01_01_DRB1_03_01_Q74R_mutant():
    # HLA-DRA*01:01/DRB1*03:01 Q74R mutant occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*03:01 Q74R mutant") is not None


def test_HLA_DRA_01_01_DRB1_04_01():
    # HLA-DRA*01:01/DRB1*04:01 occurs 241 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*04:01") is not None


def test_HLA_DRA_01_01_DRB1_04_04():
    # HLA-DRA*01:01/DRB1*04:04 occurs 891 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*04:04") is not None


def test_HLA_DRA_01_01_DRB1_07_01():
    # HLA-DRA*01:01/DRB1*07:01 occurs 873 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*07:01") is not None


def test_HLA_DRA_01_01_DRB1_08_01():
    # HLA-DRA*01:01/DRB1*08:01 occurs 1386 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*08:01") is not None


def test_HLA_DRA_01_01_DRB1_09_01():
    # HLA-DRA*01:01/DRB1*09:01 occurs 866 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*09:01") is not None


def test_HLA_DRA_01_01_DRB1_10_01():
    # HLA-DRA*01:01/DRB1*10:01 occurs 238 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*10:01") is not None


def test_HLA_DRA_01_01_DRB1_11_01():
    # HLA-DRA*01:01/DRB1*11:01 occurs 874 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*11:01") is not None


def test_HLA_DRA_01_01_DRB1_13_01():
    # HLA-DRA*01:01/DRB1*13:01 occurs 841 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*13:01") is not None


def test_HLA_DRA_01_01_DRB1_14_54():
    # HLA-DRA*01:01/DRB1*14:54 occurs 864 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*14:54") is not None


def test_HLA_DRA_01_01_DRB1_15_01():
    # HLA-DRA*01:01/DRB1*15:01 occurs 15 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB1*15:01") is not None


def test_HLA_DRA_01_01_DRB3_01_01():
    # HLA-DRA*01:01/DRB3*01:01 occurs 842 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB3*01:01") is not None


def test_HLA_DRA_01_01_DRB3_02_02():
    # HLA-DRA*01:01/DRB3*02:02 occurs 775 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB3*02:02") is not None


def test_HLA_DRA_01_01_DRB3_03_01():
    # HLA-DRA*01:01/DRB3*03:01 occurs 861 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB3*03:01") is not None


def test_HLA_DRA_01_01_DRB4_01_03():
    # HLA-DRA*01:01/DRB4*01:03 occurs 827 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB4*01:03") is not None


def test_HLA_DRA_01_01_DRB5_01_01():
    # HLA-DRA*01:01/DRB5*01:01 occurs 778 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:01/DRB5*01:01") is not None


def test_HLA_DRA_01_02_DRB1_03_01_Q74R_mutant():
    # HLA-DRA*01:02/DRB1*03:01 Q74R mutant occurs 5 times in 2019 IEDB snapshot
    assert parse("HLA-DRA*01:02/DRB1*03:01 Q74R mutant") is not None


def test_HLA_DRB1():
    # HLA-DRB1 occurs 463 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1") is not None


def test_HLA_DRB1_01_01():
    # HLA-DRB1*01:01 occurs 14358 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*01:01") is not None


def test_HLA_DRB1_01_01_D57A_mutant():
    # HLA-DRB1*01:01 D57A mutant occurs 21 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*01:01 D57A mutant") is not None


def test_HLA_DRB1_01_01_D57A_Y60H_mutant():
    # HLA-DRB1*01:01 D57A, Y60H mutant occurs 21 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*01:01 D57A, Y60H mutant") is not None


def test_HLA_DRB1_01_01_D57V_mutant():
    # HLA-DRB1*01:01 D57V mutant occurs 21 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*01:01 D57V mutant") is not None


def test_HLA_DRB1_01_01_D57V_Y60S_mutant():
    # HLA-DRB1*01:01 D57V, Y60S mutant occurs 21 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*01:01 D57V, Y60S mutant") is not None


def test_HLA_DRB1_01_01_N82A_mutant():
    # HLA-DRB1*01:01 N82A mutant occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*01:01 N82A mutant") is not None


def test_HLA_DRB1_01_02():
    # HLA-DRB1*01:02 occurs 158 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*01:02") is not None


def test_HLA_DRB1_01_03():
    # HLA-DRB1*01:03 occurs 218 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*01:03") is not None


def test_HLA_DRB1_03_01():
    # HLA-DRB1*03:01 occurs 4374 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*03:01") is not None


def test_HLA_DRB1_03_02():
    # HLA-DRB1*03:02 occurs 100 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*03:02") is not None


def test_HLA_DRB1_03_03():
    # HLA-DRB1*03:03 occurs 28 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*03:03") is not None


def test_HLA_DRB1_03_04():
    # HLA-DRB1*03:04 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*03:04") is not None


def test_HLA_DRB1_03_05():
    # HLA-DRB1*03:05 occurs 18 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*03:05") is not None


def test_HLA_DRB1_04_01():
    # HLA-DRB1*04:01 occurs 7188 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*04:01") is not None


def test_HLA_DRB1_04_02():
    # HLA-DRB1*04:02 occurs 452 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*04:02") is not None


def test_HLA_DRB1_04_03():
    # HLA-DRB1*04:03 occurs 214 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*04:03") is not None


def test_HLA_DRB1_04_04():
    # HLA-DRB1*04:04 occurs 1850 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*04:04") is not None


def test_HLA_DRB1_04_05():
    # HLA-DRB1*04:05 occurs 2456 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*04:05") is not None


def test_HLA_DRB1_04_06():
    # HLA-DRB1*04:06 occurs 94 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*04:06") is not None


def test_HLA_DRB1_04_07():
    # HLA-DRB1*04:07 occurs 10 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*04:07") is not None


def test_HLA_DRB1_04_11():
    # HLA-DRB1*04:11 occurs 4 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*04:11") is not None


def test_HLA_DRB1_07_01():
    # HLA-DRB1*07:01 occurs 4097 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*07:01") is not None


def test_HLA_DRB1_08_01():
    # HLA-DRB1*08:01 occurs 150 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*08:01") is not None


def test_HLA_DRB1_08_02():
    # HLA-DRB1*08:02 occurs 2514 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*08:02") is not None


def test_HLA_DRB1_08_03():
    # HLA-DRB1*08:03 occurs 8 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*08:03") is not None


def test_HLA_DRB1_08_04():
    # HLA-DRB1*08:04 occurs 3 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*08:04") is not None


def test_HLA_DRB1_09_01():
    # HLA-DRB1*09:01 occurs 2389 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*09:01") is not None


def test_HLA_DRB1_10_01():
    # HLA-DRB1*10:01 occurs 428 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*10:01") is not None


def test_HLA_DRB1_11_01():
    # HLA-DRB1*11:01 occurs 4287 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*11:01") is not None


def test_HLA_DRB1_11_02():
    # HLA-DRB1*11:02 occurs 70 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*11:02") is not None


def test_HLA_DRB1_11_03():
    # HLA-DRB1*11:03 occurs 67 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*11:03") is not None


def test_HLA_DRB1_11_04():
    # HLA-DRB1*11:04 occurs 140 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*11:04") is not None


def test_HLA_DRB1_12_01():
    # HLA-DRB1*12:01 occurs 1310 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*12:01") is not None


def test_HLA_DRB1_12_02():
    # HLA-DRB1*12:02 occurs 60 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*12:02") is not None


def test_HLA_DRB1_13_01():
    # HLA-DRB1*13:01 occurs 619 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*13:01") is not None


def test_HLA_DRB1_13_02():
    # HLA-DRB1*13:02 occurs 2681 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*13:02") is not None


def test_HLA_DRB1_13_03():
    # HLA-DRB1*13:03 occurs 24 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*13:03") is not None


def test_HLA_DRB1_13_04():
    # HLA-DRB1*13:04 occurs 12 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*13:04") is not None


def test_HLA_DRB1_13_05():
    # HLA-DRB1*13:05 occurs 1 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*13:05") is not None


def test_HLA_DRB1_14_01():
    # HLA-DRB1*14:01 occurs 49 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*14:01") is not None


def test_HLA_DRB1_14_02():
    # HLA-DRB1*14:02 occurs 40 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*14:02") is not None


def test_HLA_DRB1_15_01():
    # HLA-DRB1*15:01 occurs 4752 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*15:01") is not None


def test_HLA_DRB1_15_02():
    # HLA-DRB1*15:02 occurs 167 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*15:02") is not None


def test_HLA_DRB1_15_03():
    # HLA-DRB1*15:03 occurs 59 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*15:03") is not None


def test_HLA_DRB1_16_01():
    # HLA-DRB1*16:01 occurs 56 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*16:01") is not None


def test_HLA_DRB1_16_02():
    # HLA-DRB1*16:02 occurs 249 times in 2019 IEDB snapshot
    assert parse("HLA-DRB1*16:02") is not None


def test_HLA_DRB3():
    # HLA-DRB3 occurs 141 times in 2019 IEDB snapshot
    assert parse("HLA-DRB3") is not None


def test_HLA_DRB3_01_01():
    # HLA-DRB3*01:01 occurs 2270 times in 2019 IEDB snapshot
    assert parse("HLA-DRB3*01:01") is not None


def test_HLA_DRB3_02_02():
    # HLA-DRB3*02:02 occurs 1111 times in 2019 IEDB snapshot
    assert parse("HLA-DRB3*02:02") is not None


def test_HLA_DRB3_03_01():
    # HLA-DRB3*03:01 occurs 31 times in 2019 IEDB snapshot
    assert parse("HLA-DRB3*03:01") is not None


def test_HLA_DRB4():
    # HLA-DRB4 occurs 140 times in 2019 IEDB snapshot
    assert parse("HLA-DRB4") is not None


def test_HLA_DRB4_01_01():
    # HLA-DRB4*01:01 occurs 2424 times in 2019 IEDB snapshot
    assert parse("HLA-DRB4*01:01") is not None


def test_HLA_DRB4_01_03():
    # HLA-DRB4*01:03 occurs 22 times in 2019 IEDB snapshot
    assert parse("HLA-DRB4*01:03") is not None


def test_HLA_DRB5():
    # HLA-DRB5 occurs 166 times in 2019 IEDB snapshot
    assert parse("HLA-DRB5") is not None


def test_HLA_DRB5_01_01():
    # HLA-DRB5*01:01 occurs 3273 times in 2019 IEDB snapshot
    assert parse("HLA-DRB5*01:01") is not None


def test_HLA_DRB5_01_02():
    # HLA-DRB5*01:02 occurs 64 times in 2019 IEDB snapshot
    assert parse("HLA-DRB5*01:02") is not None


def test_HLA_E():
    # HLA-E occurs 35 times in 2019 IEDB snapshot
    assert parse("HLA-E") is not None


def test_HLA_E_01_01():
    # HLA-E*01:01 occurs 57 times in 2019 IEDB snapshot
    assert parse("HLA-E*01:01") is not None


def test_HLA_E_01_03():
    # HLA-E*01:03 occurs 737 times in 2019 IEDB snapshot
    assert parse("HLA-E*01:03") is not None


def test_HLA_G():
    # HLA-G occurs 16 times in 2019 IEDB snapshot
    assert parse("HLA-G") is not None


def test_HLA_G_01_01():
    # HLA-G*01:01 occurs 8 times in 2019 IEDB snapshot
    assert parse("HLA-G*01:01") is not None


def test_Mamu_class_I():
    # Mamu class I occurs 3 times in 2019 IEDB snapshot
    assert parse("Mamu class I") is not None


def test_Mamu_A1_001_01():
    # Mamu-A1*001:01 occurs 2421 times in 2019 IEDB snapshot
    assert parse("Mamu-A1*001:01") is not None


def test_Mamu_A1_002_01():
    # Mamu-A1*002:01 occurs 625 times in 2019 IEDB snapshot
    assert parse("Mamu-A1*002:01") is not None


def test_Mamu_A1_011_01():
    # Mamu-A1*011:01 occurs 960 times in 2019 IEDB snapshot
    assert parse("Mamu-A1*011:01") is not None


def test_Mamu_B_001_01():
    # Mamu-B*001:01 occurs 650 times in 2019 IEDB snapshot
    assert parse("Mamu-B*001:01") is not None


def test_Mamu_B_003_01():
    # Mamu-B*003:01 occurs 32 times in 2019 IEDB snapshot
    assert parse("Mamu-B*003:01") is not None


def test_Mamu_B_004_01():
    # Mamu-B*004:01 occurs 9 times in 2019 IEDB snapshot
    assert parse("Mamu-B*004:01") is not None


def test_Mamu_B_008_01():
    # Mamu-B*008:01 occurs 880 times in 2019 IEDB snapshot
    assert parse("Mamu-B*008:01") is not None


def test_Mamu_B_017_04():
    # Mamu-B*017:04 occurs 1458 times in 2019 IEDB snapshot
    assert parse("Mamu-B*017:04") is not None


def test_Mamu_B_065_02():
    # Mamu-B*065:02 occurs 17 times in 2019 IEDB snapshot
    assert parse("Mamu-B*065:02") is not None


def test_Mamu_DR1B():
    # Mamu-DR1B occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DR1B") is not None


def test_Mamu_DR2():
    # Mamu-DR2 occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DR2") is not None


def test_Mamu_DR3A():
    # Mamu-DR3A occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DR3A") is not None


def test_Mamu_DR4():
    # Mamu-DR4 occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DR4") is not None


def test_Mamu_DR6():
    # Mamu-DR6 occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DR6") is not None


def test_Mamu_DR7A():
    # Mamu-DR7A occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DR7A") is not None


def test_Mamu_DR8():
    # Mamu-DR8 occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DR8") is not None


def test_Mamu_DRB_w2_01():
    # Mamu-DRB*w2:01 occurs 93 times in 2019 IEDB snapshot
    assert parse("Mamu-DRB*w2:01") is not None


def test_Mamu_DRB1_03_03():
    # Mamu-DRB1*03:03 occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DRB1*03:03") is not None


def test_Mamu_DRB1_03_06():
    # Mamu-DRB1*03:06 occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DRB1*03:06") is not None


def test_Mamu_DRB1_03_08():
    # Mamu-DRB1*03:08 occurs 2 times in 2019 IEDB snapshot
    assert parse("Mamu-DRB1*03:08") is not None


def test_Mamu_DRB1_04_06():
    # Mamu-DRB1*04:06 occurs 94 times in 2019 IEDB snapshot
    assert parse("Mamu-DRB1*04:06") is not None


def test_Patr_A_01_01():
    # Patr-A*01:01 occurs 108 times in 2019 IEDB snapshot
    assert parse("Patr-A*01:01") is not None


def test_Patr_A_03_01():
    # Patr-A*03:01 occurs 8 times in 2019 IEDB snapshot
    assert parse("Patr-A*03:01") is not None


def test_Patr_A_04_01():
    # Patr-A*04:01 occurs 3 times in 2019 IEDB snapshot
    assert parse("Patr-A*04:01") is not None


def test_Patr_A_05_01():
    # Patr-A*05:01 occurs 4 times in 2019 IEDB snapshot
    assert parse("Patr-A*05:01") is not None


def test_Patr_A_06_01():
    # Patr-A*06:01 occurs 4 times in 2019 IEDB snapshot
    assert parse("Patr-A*06:01") is not None


def test_Patr_A_06_02():
    # Patr-A*06:02 occurs 41 times in 2019 IEDB snapshot
    assert parse("Patr-A*06:02") is not None


def test_Patr_A_07_01():
    # Patr-A*07:01 occurs 99 times in 2019 IEDB snapshot
    assert parse("Patr-A*07:01") is not None


def test_Patr_A_09_01():
    # Patr-A*09:01 occurs 138 times in 2019 IEDB snapshot
    assert parse("Patr-A*09:01") is not None


def test_Patr_AL():
    # Patr-AL occurs 69 times in 2019 IEDB snapshot
    assert parse("Patr-AL") is not None


def test_Patr_B_01_01():
    # Patr-B*01:01 occurs 141 times in 2019 IEDB snapshot
    assert parse("Patr-B*01:01") is not None


def test_Patr_B_09_01():
    # Patr-B*09:01 occurs 48 times in 2019 IEDB snapshot
    assert parse("Patr-B*09:01") is not None


def test_Patr_B_13_01():
    # Patr-B*13:01 occurs 128 times in 2019 IEDB snapshot
    assert parse("Patr-B*13:01") is not None


def test_Patr_B_17_01():
    # Patr-B*17:01 occurs 8 times in 2019 IEDB snapshot
    assert parse("Patr-B*17:01") is not None


def test_Patr_B_24_01():
    # Patr-B*24:01 occurs 76 times in 2019 IEDB snapshot
    assert parse("Patr-B*24:01") is not None


def test_Patr_DR1():
    # Patr-DR1 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR1") is not None


def test_Patr_DR10():
    # Patr-DR10 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR10") is not None


def test_Patr_DR2():
    # Patr-DR2 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR2") is not None


def test_Patr_DR3():
    # Patr-DR3 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR3") is not None


def test_Patr_DR4():
    # Patr-DR4 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR4") is not None


def test_Patr_DR5():
    # Patr-DR5 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR5") is not None


def test_Patr_DR7():
    # Patr-DR7 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR7") is not None


def test_Patr_DR8():
    # Patr-DR8 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR8") is not None


def test_Patr_DR9():
    # Patr-DR9 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DR9") is not None


def test_Patr_DRB1_03_08():
    # Patr-DRB1*03:08 occurs 2 times in 2019 IEDB snapshot
    assert parse("Patr-DRB1*03:08") is not None


def test_RT1_A():
    # RT1-A occurs 3 times in 2019 IEDB snapshot
    assert parse("RT1-A") is not None


def test_RT1_A1():
    # RT1-A1 occurs 6 times in 2019 IEDB snapshot
    assert parse("RT1-A1") is not None


def test_RT1_A2():
    # RT1-A2 occurs 8 times in 2019 IEDB snapshot
    assert parse("RT1-A2") is not None


def test_RT1_Aa():
    # RT1-Aa occurs 13 times in 2019 IEDB snapshot
    assert parse("RT1-Aa") is not None


def test_RT1_Ac():
    # RT1-Ac occurs 1 times in 2019 IEDB snapshot
    assert parse("RT1-Ac") is not None


def test_RT1_B():
    # RT1-B occurs 32 times in 2019 IEDB snapshot
    assert parse("RT1-B") is not None


def test_RT1_Ba():
    # RT1-Ba occurs 59 times in 2019 IEDB snapshot
    assert parse("RT1-Ba") is not None


def test_RT1_Bl():
    # RT1-Bl occurs 135 times in 2019 IEDB snapshot
    assert parse("RT1-Bl") is not None


def test_RT1_Bn():
    # RT1-Bn occurs 59 times in 2019 IEDB snapshot
    assert parse("RT1-Bn") is not None


def test_RT1_Bu():
    # RT1-Bu occurs 48 times in 2019 IEDB snapshot
    assert parse("RT1-Bu") is not None


def test_RT1_Da():
    # RT1-Da occurs 59 times in 2019 IEDB snapshot
    assert parse("RT1-Da") is not None


def test_RT1_Dl():
    # RT1-Dl occurs 61 times in 2019 IEDB snapshot
    assert parse("RT1-Dl") is not None


def test_RT1_Dn():
    # RT1-Dn occurs 59 times in 2019 IEDB snapshot
    assert parse("RT1-Dn") is not None


def test_RT1_Du():
    # RT1-Du occurs 48 times in 2019 IEDB snapshot
    assert parse("RT1-Du") is not None


def test_RT11_class_II():
    # RT11 class II occurs 2 times in 2019 IEDB snapshot
    assert parse("RT11 class II") is not None


def test_SLA_class_I():
    # SLA class I occurs 8 times in 2019 IEDB snapshot
    assert parse("SLA class I") is not None


def test_SLA_1_0401():
    # SLA-1*0401 occurs 70 times in 2019 IEDB snapshot
    assert parse("SLA-1*0401") is not None


def test_SLA_1_07_02():
    # SLA-1*07:02 occurs 15 times in 2019 IEDB snapshot
    assert parse("SLA-1*07:02") is not None


def test_SLA_1_1502():
    # SLA-1*1502 occurs 2 times in 2019 IEDB snapshot
    assert parse("SLA-1*1502") is not None


def test_SLA_2_0401():
    # SLA-2*0401 occurs 11 times in 2019 IEDB snapshot
    assert parse("SLA-2*0401") is not None


def test_SLA_3_0202():
    # SLA-3*0202 occurs 23 times in 2019 IEDB snapshot
    assert parse("SLA-3*0202") is not None


def test_YF1w_7_1():
    # YF1w*7.1 occurs 2 times in 2019 IEDB snapshot
    assert parse("YF1w*7.1") is not None


def test_cattle_CD1b3():
    # cattle CD1b3 occurs 2 times in 2019 IEDB snapshot
    assert parse("cattle CD1b3") is not None


def test_cattle_CD1d():
    # cattle CD1d occurs 1 times in 2019 IEDB snapshot
    assert parse("cattle CD1d") is not None


def test_chicken_CD1_1():
    # chicken CD1-1 occurs 3 times in 2019 IEDB snapshot
    assert parse("chicken CD1-1") is not None


def test_human_BTN3A1():
    # human BTN3A1 occurs 4 times in 2019 IEDB snapshot
    assert parse("human BTN3A1") is not None


def test_human_CD1a():
    # human CD1a occurs 8 times in 2019 IEDB snapshot
    assert parse("human CD1a") is not None


def test_human_CD1b():
    # human CD1b occurs 2 times in 2019 IEDB snapshot
    assert parse("human CD1b") is not None


def test_human_CD1c():
    # human CD1c occurs 4 times in 2019 IEDB snapshot
    assert parse("human CD1c") is not None


def test_human_CD1d():
    # human CD1d occurs 8 times in 2019 IEDB snapshot
    assert parse("human CD1d") is not None


def test_human_MR1():
    # human MR1 occurs 48 times in 2019 IEDB snapshot
    assert parse("human MR1") is not None


def test_mouse_CD1d():
    # mouse CD1d occurs 40 times in 2019 IEDB snapshot
    assert parse("mouse CD1d") is not None
